@@ -279,8 +279,15 @@ void blob_zero(Blob *pBlob){
 
 /*
 ** Append text or data to the end of a blob.
+**
+** The blob_append_full() routine is a complete implementation.
+** The blob_append() routine only works for cases where nData>0 and
+** no resizing is required, and falls back to blob_append_full() if
+** either condition is not met, but runs faster in the common case
+** where all conditions are met.  The use of blob_append() is
+** recommended, unless it is known in advance that nData<0.
 */
-void blob_append(Blob *pBlob, const char *aData, int nData){
+void blob_append_full(Blob *pBlob, const char *aData, int nData){
   sqlite3_int64 nNew;
   assert( aData!=0 || nData==0 );
   blob_is_init(pBlob);
@@ -303,18 +310,36 @@ void blob_append(Blob *pBlob, const char *aData, int nData){
   pBlob->nUsed += nData;
   pBlob->aData[pBlob->nUsed] = 0;   /* Blobs are always nul-terminated */
 }
+void blob_append(Blob *pBlob, const char *aData, int nData){
+  sqlite3_int64 nUsed;
+  assert( aData!=0 || nData==0 );
+  /* blob_is_init(pBlob); // omitted for speed */
+  if( nData<=0 || pBlob->nUsed + nData >= pBlob->nAlloc ){
+    blob_append_full(pBlob, aData, nData);
+    return;
+  }
+  nUsed = pBlob->nUsed;
+  pBlob->nUsed += nData;
+  pBlob->aData[pBlob->nUsed] = 0;
+  memcpy(&pBlob->aData[nUsed], aData, nData);
+}
+
+/*
+** Append a string literal to a blob.
+*/
+#if INTERFACE
+#define blob_append_string(BLOB,STR) blob_append(BLOB,STR,sizeof(STR)-1)
+#endif
 
 /*
 ** Append a single character to the blob
 */
 void blob_append_char(Blob *pBlob, char c){
   if( pBlob->nUsed+1 >= pBlob->nAlloc ){
-    pBlob->xRealloc(pBlob, pBlob->nUsed + pBlob->nAlloc + 100);
-    if( pBlob->nUsed + 1 >= pBlob->nAlloc ){
-      blob_panic();
-    }
+    blob_append_full(pBlob, &c, 1);
+  }else{
+    pBlob->aData[pBlob->nUsed++] = c;
   }
-  pBlob->aData[pBlob->nUsed++] = c;    
 }
 
 /*
@@ -335,12 +360,25 @@ char *blob_str(Blob *p){
     blob_append_char(p, 0); /* NOTE: Changes nUsed. */
     p->nUsed = 0;
   }
-  if( p->aData[p->nUsed]!=0 ){
+  if( p->nUsed<p->nAlloc ){
+    p->aData[p->nUsed] = 0;
+  }else{
     blob_materialize(p);
   }
   return p->aData;
 }
 
+/*
+** Compute the string length of a Blob.  If there are embedded
+** nul characters, truncate the to blob at the first nul.
+*/
+int blob_strlen(Blob *p){
+  char *z = blob_str(p);
+  if( z==0 ) return 0;
+  p->nUsed = (int)strlen(p->aData);
+  return p->nUsed;
+}
+
 /*
 ** Return a pointer to a null-terminated string for a blob that has
 ** been created using blob_append_sql() and not blob_appendf().  If
@@ -453,6 +491,30 @@ void blob_resize(Blob *pBlob, unsigned int newSize){
   pBlob->aData[newSize] = 0;
 }
 
+/*
+** Ensures that the given blob has at least the given amount of memory
+** allocated to it. Does not modify pBlob->nUsed nor will it reduce
+** the currently-allocated amount of memory.
+**
+** For semantic compatibility with blob_append_full(), if newSize is
+** >=0x7fff000 (~2GB) then this function will trigger blob_panic(). If
+** it didn't, it would be possible to bypass that hard-coded limit via
+** this function.
+**
+** We've had at least one report:
+**   https://fossil-scm.org/forum/forumpost/b7bbd28db4
+** which implies that this is unconditionally failing on mingw 32-bit
+** builds.
+*/
+void blob_reserve(Blob *pBlob, unsigned int newSize){
+  if(newSize>=0x7fff0000 ){
+    blob_panic();
+  }else if(newSize>pBlob->nUsed){
+    pBlob->xRealloc(pBlob, newSize);
+    pBlob->aData[newSize] = 0;
+  }
+}
+
 /*
 ** Make sure a blob is nul-terminated and is not a pointer to unmanaged
 ** space.  Return a pointer to the data.
@@ -1142,7 +1204,6 @@ void test_cycle_compress(void){
   fossil_print("ok\n");
 }
 
-#if defined(_WIN32) || defined(__CYGWIN__)
 /*
 ** Convert every \n character in the given blob into \r\n.
 */
@@ -1166,7 +1227,6 @@ void blob_add_cr(Blob *p){
     }
   }
 }
-#endif
 
 /*
 ** Remove every \r character from the given blob, replacing each one with
@@ -1259,21 +1319,25 @@ void blob_append_escaped_arg(Blob *pBlob, const char *zIn){
   int n = blob_size(pBlob);
   char *z = blob_buffer(pBlob);
 #if defined(_WIN32)
+  const char cDirSep = '\\';  /* Use \ as directory separator */
   const char cQuote = '"';    /* Use "..." quoting on windows */
+  const char cEscape = '^';   /* Use ^X escaping on windows */
 #else
+  const char cDirSep = '/';   /* Use / as directory separator */
   const char cQuote = '\'';   /* Use '...' quoting on unix */
+  const char cEscape = '\\';  /* Use \X escaping on unix */
 #endif
 
   for(i=0; (c = zIn[i])!=0; i++){
     if( c==cQuote || (unsigned char)c<' ' ||
-        c=='\\' || c==';' || c=='*' || c=='?' || c=='[' ){
+        c==cEscape || c==';' || c=='*' || c=='?' || c=='[' ){
       Blob bad;
       blob_token(pBlob, &bad);
       fossil_fatal("the [%s] argument to the \"%s\" command contains "
                    "a character (ascii 0x%02x) that is a security risk",
                    zIn, blob_str(&bad), c);
     }
-    if( !needEscape && !fossil_isalnum(c) && c!='/' && c!='.' && c!='_' ){
+    if( !needEscape && !fossil_isalnum(c) && c!=cDirSep && c!='.' && c!='_' ){
       needEscape = 1;
     }
   }
@@ -1281,9 +1345,58 @@ void blob_append_escaped_arg(Blob *pBlob, const char *zIn){
     blob_append_char(pBlob, ' ');
   }
   if( needEscape ) blob_append_char(pBlob, cQuote);
-  if( zIn[0]=='-' ) blob_append(pBlob, "./", 2);
+  if( zIn[0]=='-' ){
+    blob_append_char(pBlob, '.');
+    blob_append_char(pBlob, cDirSep);
+#if defined(_WIN32)
+  }else if( zIn[0]=='/' ){
+    blob_append_char(pBlob, '.');
+#endif
+  }
+#if defined(_WIN32)
+  if( needEscape ){
+    for(i=0; (c = zIn[i])!=0; i++){
+      if( c==cQuote ) blob_append_char(pBlob, cDirSep);
+      blob_append_char(pBlob, c);
+    }
+  }else{
+    blob_append(pBlob, zIn, -1);
+  }
+#else
   blob_append(pBlob, zIn, -1);
-  if( needEscape ) blob_append_char(pBlob, cQuote);
+#endif
+  if( needEscape ){
+#if defined(_WIN32)
+    /* NOTE: Trailing backslash must be doubled before final double quote. */
+    if( pBlob->aData[pBlob->nUsed-1]==cDirSep ){
+      blob_append_char(pBlob, cDirSep);
+    }
+#endif
+    blob_append_char(pBlob, cQuote);
+  }
+}
+
+/*
+** COMMAND: test-escaped-arg
+**
+** Usage %fossil ARG ...
+**
+** Run each argument through blob_append_escaped_arg() and show the
+** result.  Append each argument to "fossil test-echo" and run that
+** using fossil_system() to verify that it really does get escaped
+** correctly.
+*/
+void test_escaped_arg__cmd(void){
+  int i;
+  Blob x;
+  blob_init(&x, 0, 0);
+  for(i=2; i<g.argc; i++){
+    fossil_print("%3d [%s]: ", i, g.argv[i]);
+    blob_appendf(&x, "fossil test-echo %$", g.argv[i]);
+    fossil_print("%s\n", blob_str(&x));
+    fossil_system(blob_str(&x));
+    blob_reset(&x);
+  }
 }
 
 /*
@@ -1346,7 +1459,7 @@ void blob_to_utf8_no_bom(Blob *pBlob, int useMbcs){
     if( bomReverse ){
       /* Found BOM, but with reversed bytes */
       unsigned int i = blob_size(pBlob);
-      while( i>0 ){
+      while( i>1 ){
         /* swap bytes of unicode representation */
         char zTemp = zUtf8[--i];
         zUtf8[i] = zUtf8[i-1];
@@ -1354,9 +1467,10 @@ void blob_to_utf8_no_bom(Blob *pBlob, int useMbcs){
       }
     }
     /* Make sure the blob contains two terminating 0-bytes */
-    blob_append_char(pBlob, 0);
+    blob_append(pBlob, "\000\000", 3);
     zUtf8 = blob_str(pBlob) + bomSize;
     zUtf8 = fossil_unicode_to_utf8(zUtf8);
+    blob_reset(pBlob);
     blob_set_dynamic(pBlob, zUtf8);
   }else if( useMbcs && invalid_utf8(pBlob) ){
 #if defined(_WIN32) || defined(__CYGWIN__)
