@@ -21,6 +21,20 @@
 #include "branch.h"
 #include <assert.h>
 
+/*
+** Return true if zBr is the branch name associated with check-in with
+** blob.uuid value of zUuid
+*/
+int branch_includes_uuid(const char *zBr, const char *zUuid){
+  return db_exists(
+    "SELECT 1 FROM tagxref, blob"
+    " WHERE blob.uuid=%Q AND tagxref.rid=blob.rid"
+    "   AND tagxref.value=%Q AND tagxref.tagtype>0"
+    "   AND tagxref.tagid=%d",
+    zUuid, zBr, TAG_BRANCH
+  );
+}
+
 /*
 ** If RID refers to a check-in, return the name of the branch for that
 ** check-in.
@@ -146,7 +160,6 @@ void branch_new(void){
   blob_appendf(&branch, "T *branch * %F\n", zBranch);
   blob_appendf(&branch, "T *sym-%F *\n", zBranch);
   if( isPrivate ){
-    blob_appendf(&branch, "T +private *\n");
     noSign = 1;
   }
 
@@ -247,7 +260,7 @@ static const char createBrlistQuery[] =
 
 /* Call this routine to create the TEMP table */
 static void brlist_create_temp_table(void){
-  db_multi_exec(createBrlistQuery/*works-like:""*/);
+  db_exec_sql(createBrlistQuery);
 }
 
 
@@ -336,15 +349,15 @@ int branch_is_open(const char *zBrName){
 ** Run various subcommands to manage branches of the open repository or
 ** of the repository identified by the -R or --repository option.
 **
-**    fossil branch current
+** >  fossil branch current
 **
 **        Print the name of the branch for the current check-out
 **
-**    fossil branch info BRANCH-NAME
+** >  fossil branch info BRANCH-NAME
 **
 **        Print information about a branch
 **
-**    fossil branch list|ls ?OPTIONS?
+** >  fossil branch list|ls ?OPTIONS?
 **
 **        List all branches. Options:
 **          -a|--all      List all branches.  Default show only open branches
@@ -352,7 +365,7 @@ int branch_is_open(const char *zBrName){
 **          -r            Reverse the sort order
 **          -t            Show recently changed branches first
 **
-**    fossil branch new BRANCH-NAME BASIS ?OPTIONS?
+** >  fossil branch new BRANCH-NAME BASIS ?OPTIONS?
 **
 **        Create a new branch BRANCH-NAME off of check-in BASIS.
 **        Supported options for this subcommand include:
@@ -368,14 +381,9 @@ int branch_is_open(const char *zBrName){
 **        from UTC as "-HH:MM" (westward) or "+HH:MM" (eastward).
 **        Either no timezone suffix or "Z" means UTC.
 **
-** Options:
-**    -R|--repository FILE       Run commands on repository FILE
+** Options valid for all subcommands:
 **
-** Summary:
-**    fossil branch current
-**    fossil branch info BRANCHNAME
-**    fossil branch [list|ls]
-**    fossil branch new
+**    -R|--repository FILE       Run commands on repository FILE
 */
 void branch_cmd(void){
   int n;
@@ -452,6 +460,7 @@ static void new_brlist_page(void){
   int show_colors = PB("colors");
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
+  style_set_current_feature("branch");
   style_header("Branches");
   style_adunit_config(ADUNIT_RIGHT_OK);
   style_submenu_checkbox("colors", "Use Branch Colors", 0, 0);
@@ -508,7 +517,7 @@ static void new_brlist_page(void){
   @ </tbody></table></div>
   db_finalize(&q);
   style_table_sorter();
-  style_footer();
+  style_finish_page();
 }
 
 /*
@@ -550,6 +559,7 @@ void brlist_page(void){
   if( showAll ) brFlags = BRL_BOTH;
   if( showClosed ) brFlags = BRL_CLOSED_ONLY;
 
+  style_set_current_feature("branch");
   style_header("%s", showClosed ? "Closed Branches" :
                         showAll ? "All Branches" : "Open Branches");
   style_submenu_element("Timeline", "brtimeline");
@@ -616,7 +626,7 @@ void brlist_page(void){
     @ </ul>
   }
   db_finalize(&q);
-  style_footer();
+  style_finish_page();
 }
 
 /*
@@ -665,6 +675,7 @@ void brtimeline_page(void){
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
 
+  style_set_current_feature("branch");
   style_header("Branches");
   style_submenu_element("List", "brlist");
   login_anonymous_available();
@@ -692,5 +703,5 @@ void brtimeline_page(void){
   if( PB("ubg")!=0 ) tmFlags |= TIMELINE_UCOLOR;
   www_print_timeline(&q, tmFlags, 0, 0, 0, 0, 0, brtimeline_extra);
   db_finalize(&q);
-  style_footer();
+  style_finish_page();
 }
