@@ -1055,6 +1055,17 @@ static NORETURN void malformed_request(const char *zMsg);
 **
 ** SCGI typically omits PATH_INFO.  CGI sometimes omits REQUEST_URI and
 ** PATH_INFO when it is empty.
+**
+** CGI Parameter quick reference:
+**
+**                                      REQUEST_URI
+**                               _____________|____________
+**                              /                          \
+**    https://www.fossil-scm.org/forum/info/12736b30c072551a?t=c
+**            \________________/\____/\____________________/ \_/
+**                    |            |             |            |
+**               HTTP_HOST         |        PATH_INFO     QUERY_STRING
+**                            SCRIPT_NAME
 */
 void cgi_init(void){
   char *z;
@@ -1073,7 +1084,23 @@ void cgi_init(void){
 #endif
   g.isHTTP = 1;
   cgi_destination(CGI_BODY);
-  if( zScriptName==0 ) malformed_request("missing SCRIPT_NAME");
+
+  /* We must have SCRIPT_NAME. If the web server did not supply it, try
+  ** to compute it from REQUEST_URI and PATH_INFO. */
+  if( zScriptName==0 ){
+    size_t nRU, nPI;
+    if( zRequestUri==0 || zPathInfo==0 ){
+      malformed_request("missing SCRIPT_NAME");  /* Does not return */
+    }
+    nRU = strlen(zRequestUri);
+    nPI = strlen(zPathInfo);
+    if( nRU<nPI ){
+      malformed_request("PATH_INFO is longer than REQUEST_URI");
+    }
+    zScriptName = mprintf("%.*s", (int)(nRU-nPI), zRequestUri);
+    cgi_set_parameter("SCRIPT_NAME", zScriptName);
+  }
+
 #ifdef _WIN32
   /* The Microsoft IIS web server does not define REQUEST_URI, instead it uses
   ** PATH_INFO for virtually the same purpose.  Define REQUEST_URI the same as
@@ -1259,12 +1286,11 @@ const char *cgi_parameter(const char *zName, const char *zDefault){
 
   /* If no match is found and the name begins with an upper-case
   ** letter, then check to see if there is an environment variable
-  ** with the given name. Handle environment variables with empty values
-  ** the same as non-existent environment variables.
+  ** with the given name.
   */
   if( fossil_isupper(zName[0]) ){
     const char *zValue = fossil_getenv(zName);
-    if( zValue && zValue[0] ){
+    if( zValue ){
       cgi_set_parameter_nocopy(zName, zValue, 0);
       CGIDEBUG(("env-match [%s] = [%s]\n", zName, zValue));
       return zValue;
@@ -1403,10 +1429,10 @@ void cgi_load_environment(void){
     "HTTP_IF_NONE_MATCH", "HTTP_IF_MODIFIED_SINCE",
     "HTTP_USER_AGENT", "HTTP_REFERER", "PATH_INFO", "PATH_TRANSLATED",
     "QUERY_STRING", "REMOTE_ADDR", "REMOTE_PORT",
-    "REMOTE_USER", "REQUEST_METHOD",
-    "REQUEST_URI", "SCRIPT_FILENAME", "SCRIPT_NAME", "SERVER_PROTOCOL",
-    "HOME", "FOSSIL_HOME", "USERNAME", "USER", "FOSSIL_USER",
-    "SQLITE_TMPDIR", "TMPDIR",
+    "REMOTE_USER", "REQUEST_METHOD", "REQUEST_SCHEME",
+    "REQUEST_URI", "SCRIPT_FILENAME", "SCRIPT_NAME", "SERVER_NAME",
+    "SERVER_PROTOCOL", "HOME", "FOSSIL_HOME", "USERNAME", "USER",
+    "FOSSIL_USER", "SQLITE_TMPDIR", "TMPDIR",
     "TEMP", "TMP", "FOSSIL_VFS",
     "FOSSIL_FORCE_TICKET_MODERATION", "FOSSIL_FORCE_WIKI_MODERATION",
     "FOSSIL_TCL_PATH", "TH1_DELETE_INTERP", "TH1_ENABLE_DOCS",
@@ -1638,6 +1664,7 @@ char *cgi_remote_ip(int fd){
 void cgi_handle_http_request(const char *zIpAddr){
   char *z, *zToken;
   int i;
+  const char *zScheme = "http";
   char zLine[2000];     /* A single line of input. */
   g.fullHttpReply = 1;
   if( fgets(zLine, sizeof(zLine),g.httpIn)==0 ){
@@ -1701,8 +1728,13 @@ void cgi_handle_http_request(const char *zIpAddr){
       cgi_setenv("HTTP_COOKIE", zVal);
     }else if( fossil_strcmp(zFieldName,"https:")==0 ){
       cgi_setenv("HTTPS", zVal);
+      zScheme = "https";
     }else if( fossil_strcmp(zFieldName,"host:")==0 ){
+      char *z;
       cgi_setenv("HTTP_HOST", zVal);
+      z = strchr(zVal, ':');
+      if( z ) z[0] = 0;
+      cgi_setenv("SERVER_NAME", zVal);
     }else if( fossil_strcmp(zFieldName,"if-none-match:")==0 ){
       cgi_setenv("HTTP_IF_NONE_MATCH", zVal);
     }else if( fossil_strcmp(zFieldName,"if-modified-since:")==0 ){
@@ -1728,6 +1760,7 @@ void cgi_handle_http_request(const char *zIpAddr){
       }
     }
   }
+  cgi_setenv("REQUEST_SCHEME",zScheme);
   cgi_init();
   cgi_trace(0);
 }
@@ -2106,7 +2139,7 @@ int cgi_http_server(
       }
     }else
 #endif
-    if( system(zBrowser)<0 ){
+    if( fossil_system(zBrowser)<0 ){
       fossil_warning("cannot start browser: %s\n", zBrowser);
     }
   }
