@@ -346,9 +346,9 @@ void compute_descendants(int rid, int N){
 **
 ** Options:
 **    -R|--repository FILE       Extract info from repository FILE
-**    -W|--width <num>           Width of lines (default is to auto-detect).
-**                               Must be >20 or 0 (= no limit, resulting in a
-**                               single line per entry).
+**    -W|--width N               Width of lines (default is to auto-detect).
+**                               Must be greater than 20 or else 0 for no
+**                               limit, resulting in a one line per entry.
 **
 ** See also: [[finfo]], [[info]], [[leaves]]
 */
@@ -384,7 +384,7 @@ void descendants_cmd(void){
     " ORDER BY event.mtime DESC",
     timeline_query_for_tty()
   );
-  print_timeline(&q, 0, width, 0);
+  print_timeline(&q, 0, width, 0, 0);
   db_finalize(&q);
 }
 
@@ -406,9 +406,9 @@ void descendants_cmd(void){
 **   -c|--closed      show only closed leaves
 **   -m|--multiple    show only cases with multiple leaves on a single branch
 **   --recompute      recompute the "leaf" table in the repository DB
-**   -W|--width <num> Width of lines (default is to auto-detect). Must be
-**                    >39 or 0 (= no limit, resulting in a single line per
-**                    entry).
+**   -W|--width N     Width of lines (default is to auto-detect). Must be
+**                    more than 39 or else 0 no limit, resulting in a single
+**                    line per entry.
 **
 ** See also: [[descendants]], [[finfo]], [[info]], [[branch]]
 */
@@ -563,6 +563,7 @@ void leaves_page(void){
     style_submenu_element("Open", "%s", url_render(&url, 0, 0, 0, 0));
   }
   url_reset(&url);
+  style_set_current_feature("leaves");
   style_header("Leaves");
   login_anonymous_available();
   timeline_ss_submenu();
@@ -615,7 +616,7 @@ void leaves_page(void){
   www_print_timeline(&q, tmFlags, 0, 0, 0, 0, 0, 0);
   db_finalize(&q);
   @ <br />
-  style_footer();
+  style_finish_page();
 }
 
 #if INTERFACE
