@@ -127,7 +127,7 @@ struct DContext {
 ** function, a string is considered empty if it contains no characters
 ** -OR- it contains only NUL characters.
 */
-static int count_lines(
+int count_lines(
   const char *z,
   int n,
   int *pnLine
@@ -1568,7 +1568,7 @@ static void longestCommonSequence(
       iEYp = iEY;
     }
   }
-  if( iSXb==iEXb && (iE1-iS1)*(iE2-iS2)<400 ){
+  if( iSXb==iEXb && (sqlite3_int64)(iE1-iS1)*(iE2-iS2)<400 ){
     /* If no common sequence is found using the hashing heuristic and
     ** the input is not too big, use the expensive exact solution */
     optimalLCS(p, iS1, iE1, iS2, iE2, piSX, piEX, piSY, piEY);
@@ -1957,7 +1957,12 @@ int *text_diff(
         nDel += c.aEdit[i+1];
         nIns += c.aEdit[i+2];
       }
-      blob_appendf(pOut, "%10d %10d", nIns, nDel);
+      g.diffCnt[1] += nIns;
+      g.diffCnt[2] += nDel;
+      if( nIns+nDel ){
+        g.diffCnt[0]++;
+        blob_appendf(pOut, "%10d %10d", nIns, nDel);
+      }
     }else if( diffFlags & DIFF_SIDEBYSIDE ){
       sbsDiff(&c, pOut, pRe, diffFlags);
     }else{
@@ -2450,6 +2455,7 @@ void annotation_page(void){
   zCI = ann.aVers[0].zMUuid;
 
   /* generate the web page */
+  style_set_current_feature("annotate");
   style_header("Annotation For %h", zFilename);
   if( bBlame ){
     url_initialize(&url, "blame");
@@ -2485,9 +2491,9 @@ void annotation_page(void){
 
   @ <div id="annotation_log" style='display:%s(showLog?"block":"none");'>
   if( zOrigin ){
-    zLink = href("%R/finfo?name=%t&ci=%!S&orig=%!S",zFilename,zCI,zOrigin);
+    zLink = href("%R/finfo?name=%t&from=%!S&to=%!S",zFilename,zCI,zOrigin);
   }else{
-    zLink = href("%R/finfo?name=%t&ci=%!S",zFilename,zCI);
+    zLink = href("%R/finfo?name=%t&from=%!S",zFilename,zCI);
   }
   @ <h2>Versions of %z(zLink)%h(zFilename)</a> analyzed:</h2>
   @ <ol>
@@ -2504,17 +2510,17 @@ void annotation_page(void){
   if( !ann.bMoreToDo ){
     assert( ann.origId==0 );  /* bMoreToDo always set for a point-to-point */
     @ <h2>Origin for each line in
-    @ %z(href("%R/finfo?name=%h&ci=%!S", zFilename, zCI))%h(zFilename)</a>
+    @ %z(href("%R/finfo?name=%h&from=%!S", zFilename, zCI))%h(zFilename)</a>
     @ from check-in %z(href("%R/info/%!S",zCI))%S(zCI)</a>:</h2>
   }else if( ann.origId>0 ){
     @ <h2>Lines of
-    @ %z(href("%R/finfo?name=%h&ci=%!S", zFilename, zCI))%h(zFilename)</a>
+    @ %z(href("%R/finfo?name=%h&from=%!S", zFilename, zCI))%h(zFilename)</a>
     @ from check-in %z(href("%R/info/%!S",zCI))%S(zCI)</a>
     @ that are changed by the sequence of edits moving toward
     @ check-in %z(href("%R/info/%!S",zOrigin))%S(zOrigin)</a>:</h2>
   }else{
     @ <h2>Lines added by the %d(ann.nVers) most recent ancestors of
-    @ %z(href("%R/finfo?name=%h&ci=%!S", zFilename, zCI))%h(zFilename)</a>
+    @ %z(href("%R/finfo?name=%h&from=%!S", zFilename, zCI))%h(zFilename)</a>
     @ from check-in %z(href("%R/info/%!S",zCI))%S(zCI)</a>:</h2>
   }
   @ <pre>
@@ -2558,7 +2564,7 @@ void annotation_page(void){
 
   }
   @ </pre>
-  style_footer();
+  style_finish_page();
 }
 
 /*
