@@ -231,41 +231,6 @@ static int hasGap(const char *z, int n){
   return 0 ;
 }
 
-/*
-** Append text to pOut, adding formatting markup.  Terms that
-** have all lower-case letters are within <tt>..</tt>.  Terms
-** that have all upper-case letters are within <i>..</i>.
-*/
-static void appendMixedFont(Blob *pOut, const char *z, int n){
-  const char *zEnd = "";
-  int i = 0;
-  int j;
-  while( i<n ){
-    if( z[i]==' ' || z[i]=='=' ){
-      for(j=i+1; j<n && (z[j]==' ' || z[j]=='='); j++){}
-      blob_append(pOut, z+i, j-i);
-      i = j;
-    }else{
-      for(j=i; j<n && z[j]!=' ' && z[j]!='=' && !fossil_isalpha(z[j]); j++){}
-      if( j>=n || z[j]==' ' || z[j]=='=' ){
-        zEnd = "";
-      }else{
-        if( fossil_isupper(z[j]) && z[i]!='-' ){
-          blob_append(pOut, "<i>",3);
-          zEnd = "</i>";
-        }else{
-          blob_append(pOut, "<tt>", 4);
-          zEnd = "</tt>";
-        }
-      }
-      while( j<n && z[j]!=' ' && z[j]!='=' ){ j++; }
-      blob_appendf(pOut, "%#h", j-i, z+i);
-      if( zEnd[0] ) blob_append(pOut, zEnd, -1);
-      i = j;
-    }
-  }
-}
-
 /*
 ** Input string zIn starts with '['.  If the content is a hyperlink of the
 ** form [[...]] then return the index of the closing ']'.  Otherwise return 0.
@@ -282,13 +247,18 @@ static int help_is_link(const char *z, int n){
 }
 
 /*
-** Append text to pOut, adding hyperlink markup for [...].
+** Append text to pOut with changes:
+**
+**    *   Add hyperlink markup for [[...]]
+**    *   Escape HTML characters: < > & and "
+**    *   Change "%fossil" to just "fossil"
 */
 static void appendLinked(Blob *pOut, const char *z, int n){
   int i = 0;
   int j;
   while( i<n ){
-    if( z[i]=='[' && (j = help_is_link(z+i, n-i))>0 ){
+    char c = z[i];
+    if( c=='[' && (j = help_is_link(z+i, n-i))>0 ){
       if( i ) blob_append(pOut, z, i);
       z += i+2;
       n -= i+2;
@@ -297,6 +267,30 @@ static void appendLinked(Blob *pOut, const char *z, int n){
       z += j-1;
       n -= j-1;
       i = 0;
+    }else if( c=='%' && n-i>=7 && strncmp(z+i,"%fossil",7)==0 ){
+      if( i ) blob_append(pOut, z, i);
+      z += i+7;
+      n -= i+7;
+      blob_append(pOut, "fossil", 6);
+      i = 0;
+    }else if( c=='<' ){
+      if( i ) blob_append(pOut, z, i);
+      blob_append(pOut, "&lt;", 4);
+      z += i+1;
+      n -= i+1;
+      i = 0;
+    }else if( c=='>' ){
+      if( i ) blob_append(pOut, z, i);
+      blob_append(pOut, "&gt;", 4);
+      z += i+1;
+      n -= i+1;
+      i = 0;
+    }else if( c=='&' ){
+      if( i ) blob_append(pOut, z, i);
+      blob_append(pOut, "&amp;", 5);
+      z += i+1;
+      n -= i+1;
+      i = 0;
     }else{
       i++;
     }
@@ -304,6 +298,41 @@ static void appendLinked(Blob *pOut, const char *z, int n){
   blob_append(pOut, z, i);
 }
 
+/*
+** Append text to pOut, adding formatting markup.  Terms that
+** have all lower-case letters are within <tt>..</tt>.  Terms
+** that have all upper-case letters are within <i>..</i>.
+*/
+static void appendMixedFont(Blob *pOut, const char *z, int n){
+  const char *zEnd = "";
+  int i = 0;
+  int j;
+  while( i<n ){
+    if( z[i]==' ' || z[i]=='=' ){
+      for(j=i+1; j<n && (z[j]==' ' || z[j]=='='); j++){}
+      appendLinked(pOut, z+i, j-i);
+      i = j;
+    }else{
+      for(j=i; j<n && z[j]!=' ' && z[j]!='=' && !fossil_isalpha(z[j]); j++){}
+      if( j>=n || z[j]==' ' || z[j]=='=' ){
+        zEnd = "";
+      }else{
+        if( fossil_isupper(z[j]) && z[i]!='-' ){
+          blob_append(pOut, "<i>",3);
+          zEnd = "</i>";
+        }else{
+          blob_append(pOut, "<tt>", 4);
+          zEnd = "</tt>";
+        }
+      }
+      while( j<n && z[j]!=' ' && z[j]!='=' ){ j++; }
+      appendLinked(pOut, z+i, j-i);
+      if( zEnd[0] ) blob_append(pOut, zEnd, -1);
+      i = j;
+    }
+  }
+}
+
 /*
 ** Attempt to reformat plain-text help into HTML for display on a webpage.
 **
@@ -322,6 +351,8 @@ static void appendLinked(Blob *pOut, const char *z, int n){
 **      two spaces.
 **
 **   *  Indented text is show verbatim (<pre>...</pre>)
+**
+**   *  Lines that begin with "|" at the left margin are in <pre>...</pre>
 */
 static void help_to_html(const char *zHelp, Blob *pHtml){
   int i;
@@ -334,6 +365,7 @@ static void help_to_html(const char *zHelp, Blob *pHtml){
   int iLevel = 0;
   int isLI = 0;
   int isDT = 0;
+  int inPRE = 0;
   static const char *zEndDL = "</dl></blockquote>";
   static const char *zEndPRE = "</pre></blockquote>";
   static const char *zEndUL = "</ul>";
@@ -343,32 +375,46 @@ static void help_to_html(const char *zHelp, Blob *pHtml){
   azEnd[0] = "";
   while( zHelp[0] ){
     i = 0;
-    while( (c = zHelp[i])!=0
-        && c!='\n'
-        && c!='<'
-        && (c!='%' || strncmp(zHelp+i,"%fossil",7)!=0)
-    ){ i++; }
-    if( c=='%' ){
-      if( i ) blob_appendf(pHtml, "%#h", i, zHelp);
-      zHelp += i + 1;
-      wantBR = 1;
-      continue;
-    }else if( c=='<' ){
-      if( i ) blob_appendf(pHtml, "%#h", i, zHelp);
-      blob_append(pHtml, "&amp;", 5);
-      zHelp += i + 1;
-      continue;
+    while( (c = zHelp[i])!=0 && c!='\n' ){
+      if( c=='%' && i>2 && zHelp[i-2]==':' && strncmp(zHelp+i,"%fossil",7)==0 ){
+        appendLinked(pHtml, zHelp, i);
+        zHelp += i+1;
+        i = 0;
+        wantBR = 1;
+        continue;
+      }
+      i++;
     }
-    if( i>2 && zHelp[0]=='>' && zHelp[1]==' ' ){
-      isDT = 1;
-      for(nIndent=1; nIndent<i && zHelp[nIndent]==' '; nIndent++){}
+    if( i>2 && (zHelp[0]=='>' || zHelp[0]=='|') && zHelp[1]==' ' ){
+      if( zHelp[0]=='>' ){
+        isDT = 1;
+        for(nIndent=1; nIndent<i && zHelp[nIndent]==' '; nIndent++){}
+      }else{
+        if( !inPRE ){
+          blob_append(pHtml, "<pre>\n", -1);
+          inPRE = 1;
+        }
+      }
     }else{
+      if( inPRE ){
+        blob_append(pHtml, "</pre>\n", -1);
+        inPRE = 0;
+      }
       isDT = 0;
       for(nIndent=0; nIndent<i && zHelp[nIndent]==' '; nIndent++){}
     }
+    if( inPRE ){
+      blob_append(pHtml, zHelp+1, i);
+      zHelp += i + 1;
+      continue;
+    }
     if( nIndent==i ){
       if( c==0 ) break;
-      blob_append(pHtml, "\n", 1);
+      if( iLevel && azEnd[iLevel]==zEndPRE ){
+        /* Skip the newline at the end of a <pre> */
+      }else{
+        blob_append_char(pHtml, '\n');
+      }
       wantP = 1;
       wantBR = 0;
       zHelp += i+1;
@@ -429,11 +475,12 @@ static void help_to_html(const char *zHelp, Blob *pHtml){
         aIndent[iLevel] = x = nIndent+iDD;
         azEnd[iLevel] = zEndDD;
         appendMixedFont(pHtml, zHelp+nIndent, iDD-2);
-        blob_appendf(pHtml, "</dt><dd>%#h\n", i-x, zHelp+x);
+        blob_append(pHtml, "</dt><dd>",9);
+        appendLinked(pHtml, zHelp+x, i-x);
       }else{
         appendMixedFont(pHtml, zHelp+nIndent, i-nIndent);
-        blob_append(pHtml, "</dt>\n", 6);
       }
+      blob_append(pHtml, "</dt>\n", 6);
     }else if( wantBR ){
       appendMixedFont(pHtml, zHelp+nIndent, i-nIndent);
       blob_append(pHtml, "<br>\n", 5);
@@ -465,7 +512,7 @@ static void help_to_text(const char *zHelp, Blob *pText){
       i = -1;
       continue;
     }
-    if( c=='\n' && strncmp(zHelp+i+1,"> ",2)==0 ){
+    if( c=='\n' && (zHelp[i+1]=='>' || zHelp[i+1]=='|') && zHelp[i+2]==' ' ){
       blob_append(pText, zHelp, i+1);
       blob_append(pText, " ", 1);
       zHelp += i+2;
@@ -721,9 +768,10 @@ void help_page(void){
     int rc;
     const CmdOrPage *pCmd = 0;
 
+  style_set_current_feature("tkt");
     style_header("Help: %s", zCmd);
 
-    style_submenu_element("Command-List", "%s/help", g.zTop);
+    style_submenu_element("Command-List", "%R/help");
     rc = dispatch_name_search(zCmd, CMDFLAG_ANY|CMDFLAG_PREFIX, &pCmd);
     if( *zCmd=='/' ){
       /* Some of the webpages require query parameters in order to work.
@@ -824,7 +872,7 @@ void help_page(void){
     @ </ul></div>
 
   }
-  style_footer();
+  style_finish_page();
 }
 
 /*
@@ -836,6 +884,7 @@ void test_all_help_page(void){
   int i;
   Blob buf;
   blob_init(&buf,0,0);
+  style_set_current_feature("test");
   style_header("All Help Text");
   @ <dl>
   for(i=0; i<MX_COMMAND; i++){
@@ -875,7 +924,7 @@ void test_all_help_page(void){
   }
   @ </dl>
   blob_reset(&buf);
-  style_footer();
+  style_finish_page();
 }
 
 static void multi_column_list(const char **azWord, int nWord){
@@ -982,9 +1031,11 @@ static const char zOptions[] =
 ** These options can be used when TOPIC is present:
 **
 **    -h|--html         Format output as HTML rather than plain text
+**    -c|--commands     Restrict TOPIC search to commands
 */
 void help_cmd(void){
   int rc;
+  int mask = CMDFLAG_ANY;
   int isPage = 0;
   const char *z;
   const char *zCmdOrPage;
@@ -1030,10 +1081,13 @@ void help_cmd(void){
   isPage = ('/' == *g.argv[2]) ? 1 : 0;
   if(isPage){
     zCmdOrPage = "page";
+  }else if( find_option("commands","c",0)!=0 ){
+    mask = CMDFLAG_COMMAND;
+    zCmdOrPage = "command";
   }else{
     zCmdOrPage = "command or setting";
   }
-  rc = dispatch_name_search(g.argv[2], CMDFLAG_ANY|CMDFLAG_PREFIX, &pCmd);
+  rc = dispatch_name_search(g.argv[2], mask|CMDFLAG_PREFIX, &pCmd);
   if( rc ){
     int i, n;
     const char *az[5];
@@ -1043,15 +1097,16 @@ void help_cmd(void){
       fossil_print("ambiguous %s prefix: %s\n",
                  zCmdOrPage, g.argv[2]);
     }
-    fossil_print("Did you mean one of:\n");
+    fossil_print("Did you mean one of these TOPICs:\n");
     n = dispatch_approx_match(g.argv[2], 5, az);
     for(i=0; i<n; i++){
       fossil_print("  *  %s\n", az[i]);
     }
     fossil_print("Also consider using:\n");
-    fossil_print("   fossil help -a     ;# show all commands\n");
-    fossil_print("   fossil help -w     ;# show all web-pages\n");
-    fossil_print("   fossil help -s     ;# show all settings\n");
+    fossil_print("   fossil help TOPIC     ;# show help on TOPIC\n");
+    fossil_print("   fossil help -a        ;# show all commands\n");
+    fossil_print("   fossil help -w        ;# show all web-pages\n");
+    fossil_print("   fossil help -s        ;# show all settings\n");
     fossil_exit(1);
   }
   z = pCmd->zHelp;
