@@ -40,15 +40,19 @@ const char *mimetype_from_content(Blob *pBlob){
   /* A table of mimetypes based on file content prefixes
   */
   static const struct {
-    const char *zPrefix;       /* The file prefix */
-    const int size;            /* Length of the prefix */
+    const char *z;             /* Identifying file text */
+    const unsigned char sz1;   /* Length of the prefix */
+    const unsigned char of2;   /* Offset to the second segment */
+    const unsigned char sz2;   /* Size of the second segment */
+    const unsigned char mn;    /* Minimum size of input */
     const char *zMimetype;     /* The corresponding mimetype */
   } aMime[] = {
-    { "GIF87a",                  6, "image/gif"  },
-    { "GIF89a",                  6, "image/gif"  },
-    { "\211PNG\r\n\032\n",       8, "image/png"  },
-    { "\377\332\377",            3, "image/jpeg" },
-    { "\377\330\377",            3, "image/jpeg" },
+    { "GIF87a",                  6, 0, 0, 6,  "image/gif"  },
+    { "GIF89a",                  6, 0, 0, 6,  "image/gif"  },
+    { "\211PNG\r\n\032\n",       8, 0, 0, 8,  "image/png"  },
+    { "\377\332\377",            3, 0, 0, 3,  "image/jpeg" },
+    { "\377\330\377",            3, 0, 0, 3,  "image/jpeg" },
+    { "RIFFWAVEfmt",             4, 8, 7, 15, "sound/wav"  },
   };
 
   if( !looks_like_binary(pBlob) ) {
@@ -57,9 +61,14 @@ const char *mimetype_from_content(Blob *pBlob){
   x = (const unsigned char*)blob_buffer(pBlob);
   n = blob_size(pBlob);
   for(i=0; i<count(aMime); i++){
-    if( n>=aMime[i].size && memcmp(x, aMime[i].zPrefix, aMime[i].size)==0 ){
-      return aMime[i].zMimetype;
+    if( n<aMime[i].mn ) continue;
+    if( memcmp(x, aMime[i].z, aMime[i].sz1)!=0 ) continue;
+    if( aMime[i].sz2
+     && memcmp(x+aMime[i].of2, aMime[i].z+aMime[i].sz1, aMime[i].sz2)!=0
+    ){
+      continue;
     }
+    return aMime[i].zMimetype;
   }
   return "unknown/unknown";
 }
@@ -192,6 +201,7 @@ static const struct {
   { "pgm",        3, "image/x-portable-graymap"          },
   { "pgn",        3, "application/x-chess-pgn"           },
   { "pgp",        3, "application/pgp"                   },
+  { "pikchr",     6, "text/x-pikchr"                     },
   { "pl",         2, "application/x-perl"                },
   { "pm",         2, "application/x-perl"                },
   { "png",        3, "image/png"                         },
@@ -400,6 +410,24 @@ static const char *mimetype_from_name_custom(const char *zSuffix){
   return 0;
 }
 
+/*
+** Emit Javascript which applies (or optionally can apply) to both the
+** /doc and /wiki pages. None of this implements required
+** functionality, just nice-to-haves. Any calls after the first are
+** no-ops.
+*/
+void document_emit_js(void){
+  static int once = 0;
+  if(0==once++){
+    builtin_fossil_js_bundle_or("pikchr", NULL);
+    style_script_begin(__FILE__,__LINE__);
+    CX("window.addEventListener('load', "
+       "()=>window.fossil.pikchr.addSrcView(), "
+       "false);\n");
+    style_script_end();
+  }
+}
+
 /*
 ** Guess the mime-type of a document based on its name.
 */
@@ -538,7 +566,7 @@ void mimetype_list_page(void){
   }
   @ </tbody></table>
   style_table_sorter();
-  style_footer();
+  style_finish_page();
 }
 
 /*
@@ -752,7 +780,8 @@ void document_render(
       style_header("%s", zDefaultTitle);
       wiki_convert(pBody, 0, WIKI_BUTTONS);
     }
-    style_footer();
+    document_emit_js();
+    style_finish_page();
   }else if( fossil_strcmp(zMime, "text/x-markdown")==0 ){
     Blob tail = BLOB_INITIALIZER;
     markdown_to_html(pBody, &title, &tail);
@@ -762,19 +791,27 @@ void document_render(
       style_header("%s", zDefaultTitle);
     }
     convert_href_and_output(&tail);
-    style_footer();
+    document_emit_js();
+    style_finish_page();
   }else if( fossil_strcmp(zMime, "text/plain")==0 ){
     style_header("%s", zDefaultTitle);
     @ <blockquote><pre>
     @ %h(blob_str(pBody))
     @ </pre></blockquote>
-    style_footer();
+    document_emit_js();
+    style_finish_page();
   }else if( fossil_strcmp(zMime, "text/html")==0
             && doc_is_embedded_html(pBody, &title) ){
     if( blob_size(&title)==0 ) blob_append(&title,zFilename,-1);
     style_header("%s", blob_str(&title));
     convert_href_and_output(pBody);
-    style_footer();
+    document_emit_js();
+    style_finish_page();
+  }else if( fossil_strcmp(zMime, "text/x-pikchr")==0 ){
+    style_adunit_config(ADUNIT_RIGHT_OK);
+    style_header("%s", zDefaultTitle);
+    wiki_render_by_mimetype(pBody, zMime);
+    style_finish_page();
 #ifdef FOSSIL_ENABLE_TH1_DOCS
   }else if( Th_AreDocsEnabled() &&
             fossil_strcmp(zMime, "application/x-th1")==0 ){
@@ -794,7 +831,8 @@ void document_render(
       Th_Render(blob_str(pBody));
     }
     if( !raw ){
-      style_footer();
+      document_emit_js();
+      style_finish_page();
     }
 #endif
   }else{
@@ -879,6 +917,7 @@ void doc_page(void){
 
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
+  style_set_current_feature("doc");
   blob_init(&title, 0, 0);
   zDfltTitle = isUV ? "" : "Documentation";
   db_begin_transaction();
@@ -1002,7 +1041,7 @@ doc_not_found:
   if( fossil_strcmp(zCheckin,"ckout")!=0 ){
     @ in %z(href("%R/tree?ci=%T",zCheckin))%h(zCheckin)</a>
   }
-  style_footer();
+  style_finish_page();
   return;
 }
 
@@ -1175,5 +1214,5 @@ void doc_search_page(void){
   login_check_credentials();
   style_header("Document Search");
   search_screen(SRCH_DOC, 0);
-  style_footer();
+  style_finish_page();
 }
