@@ -112,10 +112,11 @@ void event_page(void){
     }
   }
   db_finalize(&q1);
+  style_set_current_feature("event");
   if( rid==0 || (specRid!=0 && specRid!=rid) ){
     style_header("No Such Tech-Note");
     @ Cannot locate a technical note called <b>%h(zId)</b>.
-    style_footer();
+    style_finish_page();
     return;
   }
   zUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", rid);
@@ -230,7 +231,8 @@ void event_page(void){
                        " WHERE tagname GLOB 'event-%q*'",
                     zId);
   attachment_list(zFullId, "<hr /><h2>Attachments:</h2><ul>");
-  style_footer();
+  document_emit_js();
+  style_finish_page();
   manifest_destroy(pTNote);
 }
 
@@ -416,6 +418,7 @@ void eventedit_page(void){
     login_needed(g.anon.Write && (rid ? g.anon.WrWiki : g.anon.NewWiki));
     return;
   }
+  style_set_current_feature("event");
 
   /* Figure out the color */
   if( rid ){
@@ -474,7 +477,7 @@ void eventedit_page(void){
       style_header("Error");
       @ Internal error:  Fossil tried to make an invalid artifact for
       @ the edited technote.
-      style_footer();
+      style_finish_page();
       return;
     }
     cgi_redirectf("%R/technote?name=%T", zId);
@@ -568,7 +571,7 @@ void eventedit_page(void){
   }
   @ </td></tr></table>
   @ </div></form>
-  style_footer();
+  style_finish_page();
 }
 
 /*
