@@ -295,9 +295,11 @@ void login_set_user_cookie(
   cgi_set_cookie(zCookieName, zCookie, login_cookie_path(),
                  bSessionCookie ? 0 : expires);
   record_login_attempt(zUsername, zIpAddr, 1);
+  db_unprotect(PROTECT_USER);
   db_multi_exec("UPDATE user SET cookie=%Q,"
                 "  cexpire=julianday('now')+%d/86400.0 WHERE uid=%d",
                 zHash, expires, uid);
+  db_protect_pop();
   fossil_free(zHash);
   if( zDest ){
     *zDest = zCookie;
@@ -358,10 +360,12 @@ void login_clear_login_data(){
     /* To logout, change the cookie value to an empty string */
     cgi_set_cookie(cookie, "",
                    login_cookie_path(), -86400);
+    db_unprotect(PROTECT_USER);
     db_multi_exec("UPDATE user SET cookie=NULL, ipaddr=NULL, "
                   "  cexpire=0 WHERE uid=%d"
                   "  AND login NOT IN ('anonymous','nobody',"
                   "  'developer','reader')", g.userUid);
+    db_protect_pop();
     cgi_replace_parameter(cookie, NULL);
     cgi_replace_parameter("anon", NULL);
   }
@@ -582,10 +586,12 @@ void login_page(void){
         char *zNewPw = sha1_shared_secret(zNew1, g.zLogin, 0);
         char *zChngPw;
         char *zErr;
+        int rc;
+
+        db_unprotect(PROTECT_USER);
         db_multi_exec(
            "UPDATE user SET pw=%Q WHERE uid=%d", zNewPw, g.userUid
         );
-        fossil_free(zNewPw);
         zChngPw = mprintf(
            "UPDATE user"
            "   SET pw=shared_secret(%Q,%Q,"
@@ -593,7 +599,10 @@ void login_page(void){
            " WHERE login=%Q",
            zNew1, g.zLogin, g.zLogin
         );
-        if( login_group_sql(zChngPw, "<p>", "</p>\n", &zErr) ){
+        fossil_free(zNewPw);
+        rc = login_group_sql(zChngPw, "<p>", "</p>\n", &zErr);
+        db_protect_pop();
+        if( rc ){
           zErrMsg = mprintf("<span class=\"loginError\">%s</span>", zErr);
           fossil_free(zErr);
         }else{
@@ -636,6 +645,7 @@ void login_page(void){
          @ </span></p>
       ;
       record_login_attempt(zUsername, zIpAddr, 0);
+      cgi_set_status(401, "Unauthorized");
     }else{
       /* Non-anonymous login is successful.  Set a cookie of the form:
       **
@@ -648,6 +658,7 @@ void login_page(void){
       redirect_to_g();
     }
   }
+  style_set_current_feature("login");
   style_header("Login/Logout");
   style_adunit_config(ADUNIT_OFF);
   @ %s(zErrMsg)
@@ -768,6 +779,7 @@ void login_page(void){
       @ for user <b>%h(g.zLogin)</b></p>
     }
     if( g.perm.Password ){
+      char *zRPW = fossil_random_password(12);
       @ <hr>
       @ <p>Change Password for user <b>%h(g.zLogin)</b>:</p>
       form_begin(0, "%R/login");
@@ -777,7 +789,7 @@ void login_page(void){
       @ size="30"/></td></tr>
       @ <tr><td class="form_label" id="newpw">New Password:</td>
       @ <td><input aria-labelledby="newpw" type="password" name="n1" \
-      @ size="30" /></td></tr>
+      @ size="30" /> Suggestion: %z(zRPW)</td></tr>
       @ <tr><td class="form_label" id="reppw">Repeat New Password:</td>
       @ <td><input aria-labledby="reppw" type="password" name="n2" \
       @ size="30" /></td></tr>
@@ -787,7 +799,7 @@ void login_page(void){
       @ </form>
     }
   }
-  style_footer();
+  style_finish_page();
 }
 
 /*
@@ -837,12 +849,14 @@ static int login_transfer_credentials(
     pStmt = 0;
     rc = sqlite3_prepare_v2(pOther, zSQL, -1, &pStmt, 0);
     if( rc==SQLITE_OK && sqlite3_step(pStmt)==SQLITE_ROW ){
+      db_unprotect(PROTECT_USER);
       db_multi_exec(
         "UPDATE user SET cookie=%Q, cexpire=%.17g"
         " WHERE login=%Q",
         zHash, 
         sqlite3_column_double(pStmt, 0), zLogin
       );
+      db_protect_pop();
       nXfer++;
     }
     sqlite3_finalize(pStmt);
@@ -1067,6 +1081,25 @@ void login_check_credentials(void){
     uid = login_basic_authentication(zIpAddr);
   }
 
+  /* Check for magic query parameters "resid" (for the username) and
+  ** "token" for the password.  Both values (if they exist) will be
+  ** obfuscated.
+  */
+  if( uid==0 ){
+    char *zUsr, *zPW;
+    if( (zUsr = unobscure(P("resid")))!=0
+     && (zPW = unobscure(P("token")))!=0
+    ){
+      char *zSha1Pw = sha1_shared_secret(zPW, zUsr, 0);
+      uid = db_int(0, "SELECT uid FROM user"
+                      " WHERE login=%Q"
+                      " AND (constant_time_cmp(pw,%Q)=0"
+                      "      OR constant_time_cmp(pw,%Q)=0)",
+                      zUsr, zSha1Pw, zPW);
+      fossil_free(zSha1Pw);
+    }
+  }
+
   /* If no user found yet, try to log in as "nobody" */
   if( uid==0 ){
     uid = db_int(0, "SELECT uid FROM user WHERE login='nobody'");
@@ -1219,7 +1252,7 @@ void login_set_capabilities(const char *zCap, unsigned flags){
                              p->TktFmt = p->Attach = p->ApndTkt =
                              p->ModWiki = p->ModTkt =
                              p->RdForum = p->WrForum = p->ModForum =
-                             p->WrTForum = p->AdminForum =
+                             p->WrTForum = p->AdminForum = p->Chat = 
                              p->EmailAlert = p->Announce = p->Debug = 1;
                              /* Fall thru into Read/Write */
       case 'i':   p->Read = p->Write = 1;                      break;
@@ -1256,6 +1289,7 @@ void login_set_capabilities(const char *zCap, unsigned flags){
 
       case '7':   p->EmailAlert = 1;                           break;
       case 'A':   p->Announce = 1;                             break;
+      case 'C':   p->Chat = 1;                                 break;
       case 'D':   p->Debug = 1;                                break;
 
       /* The "u" privilege recursively
@@ -1339,6 +1373,7 @@ int login_has_capability(const char *zCap, int nCap, u32 flgs){
       case '6':  rc = p->AdminForum;break;
       case '7':  rc = p->EmailAlert;break;
       case 'A':  rc = p->Announce;  break;
+      case 'C':  rc = p->Chat;      break;
       case 'D':  rc = p->Debug;     break;
       default:   rc = 0;            break;
     }
@@ -1414,12 +1449,16 @@ void login_needed(int anonOk){
   {
     const char *zUrl = PD("REQUEST_URI", "index");
     const char *zQS = P("QUERY_STRING");
+    char *zUrlNoQS;
+    int i;
     Blob redir;
     blob_init(&redir, 0, 0);
+    for(i=0; zUrl[i] && zUrl[i]!='?'; i++){}
+    zUrlNoQS = fossil_strndup(zUrl, i);
     if( fossil_wants_https(1) ){
-      blob_appendf(&redir, "%s/login?g=%T", g.zHttpsURL, zUrl);
+      blob_appendf(&redir, "%s/login?g=%T", g.zHttpsURL, zUrlNoQS);
     }else{
-      blob_appendf(&redir, "%R/login?g=%T", zUrl);
+      blob_appendf(&redir, "%R/login?g=%T", zUrlNoQS);
     }
     if( zQS && zQS[0] ){
       blob_appendf(&redir, "%%3f%T", zQS);
@@ -1535,7 +1574,7 @@ void register_page(void){
     style_header("Registration not possible");
     @ <p>This project does not allow user self-registration. Please contact the
     @ project administrator to obtain an account.</p>
-    style_footer();
+    style_finish_page();
     return;
   }
   zPerms = db_get("default-perms", "u");
@@ -1621,7 +1660,9 @@ void register_page(void){
        "'%q <%q>\nself-register from ip %q on '||datetime('now'),now())",
        zUserID, zPass, zStartPerms, zDName, zEAddr, g.zIpAddr);
     fossil_free(zPass);
+    db_unprotect(PROTECT_USER);
     db_multi_exec("%s", blob_sql_text(&sql));
+    db_protect_pop();
     uid = db_int(0, "SELECT uid FROM user WHERE login=%Q", zUserID);
     login_set_user_cookie(zUserID, uid, NULL, 0);
     if( doAlerts ){
@@ -1692,7 +1733,7 @@ void register_page(void){
       if( zGoto ){
         @ <p><a href='%h(zGoto)'>Continue</a>
       }
-      style_footer();
+      style_finish_page();
       return;
     }
     redirect_to_g();
@@ -1752,7 +1793,13 @@ void register_page(void){
   @ <tr>
   @   <td class="form_label" align="right" id="pswd">Password:</td>
   @   <td><input aria-labelledby="pswd" type="password" name="p" \
-  @ value="%h(zPasswd)" size="30"></td>
+  @ value="%h(zPasswd)" size="30"> \
+  if( zPasswd[0]==0 ){
+    char *zRPW = fossil_random_password(12);
+    @ Password suggestion: %z(zRPW)</td>
+  }else{
+    @ </td>
+  }
   @ <tr>
   if( iErrLine==4 ){
     @ <tr><td><td><span class='loginError'>&uarr; %h(zErr)</span></td></tr>
@@ -1784,7 +1831,7 @@ void register_page(void){
   @ Enter this 8-letter code in the "Captcha" box above.
   @ </td></tr></table></div>
   @ </form>
-  style_footer();
+  style_finish_page();
 
   free(zCaptcha);
 }
@@ -1834,10 +1881,12 @@ int login_group_sql(
     if( file_size(zRepoName, ExtFILE)<0 ){
       /* Silently remove non-existent repositories from the login group. */
       const char *zLabel = db_column_text(&q, 0);
+      db_unprotect(PROTECT_CONFIG);
       db_multi_exec(
          "DELETE FROM config WHERE name GLOB 'peer-*-%q'",
          &zLabel[10]
       );
+      db_protect_pop();
       continue;
     }
     rc = sqlite3_open_v2(
@@ -1972,6 +2021,7 @@ void login_group_join(
   zSelfProjCode = abbreviated_project_code(zSelfProjCode);
   zOtherProjCode = abbreviated_project_code(zOtherProjCode);
   db_begin_transaction();
+  db_unprotect(PROTECT_CONFIG);
   db_multi_exec(
     "DELETE FROM \"%w\".config WHERE name GLOB 'peer-*';"
     "INSERT INTO \"%w\".config(name,value) VALUES('peer-repo-%q',%Q);"
@@ -1995,6 +2045,7 @@ void login_group_join(
     "   WHERE name GLOB 'peer-*' OR name GLOB 'login-group-*'",
     zSelf
   );
+  db_protect_pop();
   db_end_transaction(0);
   db_multi_exec("DETACH other");
 
@@ -2006,7 +2057,9 @@ void login_group_join(
     "COMMIT;",
     zSelfProjCode, zSelfLabel, zSelfProjCode, zSelfRepo
   );
+  db_unprotect(PROTECT_CONFIG);
   login_group_sql(zSql, "<li> ", "</li>", pzErrMsg);
+  db_protect_pop();
   fossil_free(zSql);
 }
 
@@ -2027,6 +2080,7 @@ void login_group_leave(char **pzErrMsg){
     zProjCode
   );
   fossil_free(zProjCode);
+  db_unprotect(PROTECT_CONFIG);
   login_group_sql(zSql, "<li> ", "</li>", pzErrMsg);
   fossil_free(zSql);
   db_multi_exec(
@@ -2034,6 +2088,7 @@ void login_group_leave(char **pzErrMsg){
     " WHERE name GLOB 'peer-*'"
     "    OR name GLOB 'login-group-*';"
   );
+  db_protect_pop();
 }
 
 /*
