@@ -184,6 +184,49 @@ int start_of_branch(int rid, int eType){
   return ans;
 }
 
+/*
+** Find the RID of the most recent object with symbolic tag zTag
+** and having a type that matches zType.
+**
+** Return 0 if there are no matches.
+**
+** This is a tricky query to do efficiently.  
+** If the tag is very common (ex: "trunk") then
+** we want to use the query identified below as Q1 - which searching
+** the most recent EVENT table entries for the most recent with the tag.
+** But if the tag is relatively scarce (anything other than "trunk", basically)
+** then we want to do the indexed search show below as Q2.
+*/
+static int most_recent_event_with_tag(const char *zTag, const char *zType){
+  return db_int(0,
+    "SELECT objid FROM ("
+      /* Q1:  Begin by looking for the tag in the 30 most recent events */
+      "SELECT objid"
+       " FROM (SELECT * FROM event ORDER BY mtime DESC LIMIT 30) AS ex"
+      " WHERE type GLOB '%q'"
+        " AND EXISTS(SELECT 1 FROM tagxref, tag"
+                     " WHERE tag.tagname='sym-%q'"
+                       " AND tagxref.tagid=tag.tagid"
+                       " AND tagxref.tagtype>0"
+                       " AND tagxref.rid=ex.objid)"
+      " ORDER BY mtime DESC LIMIT 1"
+    ") UNION ALL SELECT * FROM ("
+      /* Q2: If the tag is not found in the 30 most recent events, then using
+      ** the tagxref table to index for the tag */
+      "SELECT event.objid"
+       " FROM tag, tagxref, event"
+      " WHERE tag.tagname='sym-%q'"
+        " AND tagxref.tagid=tag.tagid"
+        " AND tagxref.tagtype>0"
+        " AND event.objid=tagxref.rid"
+        " AND event.type GLOB '%q'"
+      " ORDER BY event.mtime DESC LIMIT 1"
+    ") LIMIT 1;",
+    zType, zTag, zTag, zType
+  );
+}
+
+
 /*
 ** Convert a symbolic name into a RID.  Acceptable forms:
 **
@@ -227,7 +270,6 @@ int symbolic_name_to_rid(const char *zTag, const char *zType){
   const char *zXTag;     /* zTag with optional [...] removed */
   int nXTag;             /* Size of zXTag */
   const char *zDate;     /* Expanded date-time string */
-  const char *zTagPrefix = "sym";
 
   if( zType==0 || zType[0]==0 ){
     zType = "*";
@@ -303,15 +345,7 @@ int symbolic_name_to_rid(const char *zTag, const char *zType){
 
   /* "tag:" + symbolic-name */
   if( memcmp(zTag, "tag:", 4)==0 ){
-    rid = db_int(0,
-       "SELECT event.objid, max(event.mtime)"
-       "  FROM tag, tagxref, event"
-       " WHERE tag.tagname='sym-%q' "
-       "   AND tagxref.tagid=tag.tagid AND tagxref.tagtype>0 "
-       "   AND event.objid=tagxref.rid "
-       "   AND event.type GLOB '%q'",
-       &zTag[4], zType
-    );
+    rid = most_recent_event_with_tag(&zTag[4], zType);
     if( startOfBranch ) rid = start_of_branch(rid,1);
     return rid;
   }
@@ -398,18 +432,18 @@ int symbolic_name_to_rid(const char *zTag, const char *zType){
   }
 
   if( zType[0]=='w' ){
-    zTagPrefix = "wiki";
+    rid = db_int(0,
+      "SELECT event.objid, max(event.mtime)"
+      "  FROM tag, tagxref, event"
+      " WHERE tag.tagname='wiki-%q' "
+      "   AND tagxref.tagid=tag.tagid AND tagxref.tagtype>0 "
+      "   AND event.objid=tagxref.rid "
+      "   AND event.type GLOB '%q'",
+      zTag, zType
+    );
+  }else{
+    rid = most_recent_event_with_tag(zTag, zType);
   }
-  /* Symbolic name */
-  rid = db_int(0,
-    "SELECT event.objid, max(event.mtime)"
-    "  FROM tag, tagxref, event"
-    " WHERE tag.tagname='%q-%q' "
-    "   AND tagxref.tagid=tag.tagid AND tagxref.tagtype>0 "
-    "   AND event.objid=tagxref.rid "
-    "   AND event.type GLOB '%q'",
-    zTagPrefix, zTag, zType
-  );
 
   if( rid>0 ){
     if( startOfBranch ) rid = start_of_branch(rid,1);
@@ -670,7 +704,7 @@ void ambiguous_page(void){
   }
   @ </ol>
   db_finalize(&q);
-  style_footer();
+  style_finish_page();
 }
 
 /*
@@ -690,7 +724,7 @@ int name_to_rid_www(const char *zParamName){
   if( zName==0 || zName[0]==0 ) return 0;
   rid = symbolic_name_to_rid(zName, "*");
   if( rid<0 ){
-    cgi_redirectf("%s/ambiguous/%T?src=%t", g.zTop, zName, g.zPath);
+    cgi_redirectf("%R/ambiguous/%T?src=%t", zName, g.zPath);
     rid = 0;
   }
   return rid;
@@ -778,6 +812,7 @@ void whatis_rid(int rid, int verboseFlag){
                  db_column_text(&q, 1));
     fossil_print("comment:    ");
     comment_print(db_column_text(&q,3), 0, 12, -1, get_comment_format());
+    cnt++;
   }
   db_finalize(&q);
 
@@ -800,6 +835,7 @@ void whatis_rid(int rid, int verboseFlag){
       db_column_text(&q, 2));
     fossil_print("            ");
     comment_print(db_column_text(&q,4), 0, 12, -1, get_comment_format());
+    cnt++;
   }
   db_finalize(&q);
 
@@ -835,8 +871,22 @@ void whatis_rid(int rid, int verboseFlag){
                  db_column_text(&q,2), db_column_text(&q,3));
     fossil_print("            ");
     comment_print(db_column_text(&q,1), 0, 12, -1, get_comment_format());
+    cnt++;
   }
   db_finalize(&q);
+
+  /* If other information available, try to describe the object */
+  if( cnt==0 ){
+    char *zWhere = mprintf("=%d", rid);
+    char *zDesc;
+    describe_artifacts(zWhere);
+    free(zWhere);
+    zDesc = db_text(0,
+       "SELECT printf('%%-12s%%s %%s',type||':',summary,substr(ref,1,16))"
+       "  FROM description WHERE rid=%d", rid);
+    fossil_print("%s\n", zDesc);
+    fossil_free(zDesc);
+  }
 }
 
 /*
@@ -1043,7 +1093,8 @@ static void describe_unknown_artifacts(){
     "   WHERE description.summary='unknown'\n"
     "     AND tagxref.tagid=(SELECT tagid FROM tag WHERE tagname='cluster')\n"
     "     AND blob.rid=tagxref.rid\n"
-    "     AND content(blob.uuid) GLOB ('*M '||blob.uuid||'*');"
+    "     AND CAST(content(blob.uuid) AS text)"
+    "                   GLOB ('*M '||description.uuid||'*');"
   );
 }
 
@@ -1220,7 +1271,7 @@ int describe_artifacts_to_stdout(const char *zWhere, const char *zLabel){
   int cnt = 0;
   if( zWhere!=0 ) describe_artifacts(zWhere);
   db_prepare(&q,
-    "SELECT uuid, summary, isPrivate\n"
+    "SELECT uuid, summary, coalesce(ref,''), isPrivate\n"
     "  FROM description\n"
     " ORDER BY ctime, type;"
   );
@@ -1229,8 +1280,9 @@ int describe_artifacts_to_stdout(const char *zWhere, const char *zLabel){
       fossil_print("%s\n", zLabel);
       zLabel = 0;
     }
-    fossil_print("  %.16s %s", db_column_text(&q,0), db_column_text(&q,1));
-    if( db_column_int(&q,2) ) fossil_print(" (private)");
+    fossil_print("  %.16s %s %s", db_column_text(&q,0),
+           db_column_text(&q,1), db_column_text(&q,2));
+    if( db_column_int(&q,3) ) fossil_print(" (private)");
     fossil_print("\n");
     cnt++;
   }
@@ -1312,7 +1364,7 @@ void bloblist_page(void){
       @ %d(i)..%d(i+n-1<mx?i+n-1:mx)</a>
     }
     @ </ul>
-    style_footer();
+    style_finish_page();
     return;
   }
   if( phantomOnly || privOnly || mx>n ){
@@ -1391,7 +1443,7 @@ void bloblist_page(void){
   }
   @ </table>
   db_finalize(&q);
-  style_footer();
+  style_finish_page();
 }
 
 /*
@@ -1400,27 +1452,39 @@ void bloblist_page(void){
 void table_of_public_phantoms(void){
   Stmt q;
   char *zRange;
+  double rNow;
   zRange = mprintf("IN (SELECT rid FROM phantom EXCEPT"
                    " SELECT rid FROM private)");
   describe_artifacts(zRange);
   fossil_free(zRange);
   db_prepare(&q,
-    "SELECT rid, uuid, summary, ref"
+    "SELECT rid, uuid, summary, ref,"
+    "  (SELECT mtime FROM blob, rcvfrom"
+    "    WHERE blob.uuid=ref AND rcvfrom.rcvid=blob.rcvid)"
     "  FROM description ORDER BY rid"
   );
+  rNow = db_double(0.0, "SELECT julianday('now')");
   @ <table cellpadding="2" cellspacing="0" border="1">
-  @ <tr><th>RID<th>Description<th>Source
+  @ <tr><th>RID<th>Description<th>Source<th>Age
   while( db_step(&q)==SQLITE_ROW ){
     int rid = db_column_int(&q,0);
     const char *zUuid = db_column_text(&q, 1);
     const char *zDesc = db_column_text(&q, 2);
     const char *zRef = db_column_text(&q,3);
+    double mtime = db_column_double(&q,4);
     @ <tr><td valign="top">%d(rid)</td>
     @ <td valign="top" align="left">%h(zUuid)<br>%h(zDesc)</td>
     if( zRef && zRef[0] ){
       @ <td valign="top">%z(href("%R/info/%!S",zRef))%!S(zRef)</a>
+      if( mtime>0 ){
+        char *zAge = human_readable_age(rNow - mtime);
+        @ <td valign="top">%h(zAge)
+        fossil_free(zAge);
+      }else{
+        @ <td>&nbsp;
+      }
     }else{
-      @ <td>&nbsp;
+      @ <td>&nbsp;<td>&nbsp;
     }
     @ </tr>
   }
@@ -1461,7 +1525,7 @@ void phantom_list_page(void){
     style_submenu_element("Artifact Stats", "artifact_stats");
   }
   table_of_public_phantoms();
-  style_footer();
+  style_finish_page();
 }
 
 /*
@@ -1525,7 +1589,7 @@ void bigbloblist_page(void){
   @ </tbody></table>
   db_finalize(&q);
   style_table_sorter();
-  style_footer();
+  style_finish_page();
 }
 
 /*
@@ -1649,5 +1713,5 @@ void hash_collisions_webpage(void){
                    " ORDER BY 1");
   @ <h1>Hash Prefix Collisions on All Artifacts</h1>
   collision_report("SELECT uuid FROM blob ORDER BY 1");
-  style_footer();
+  style_finish_page();
 }
