@@ -54,6 +54,7 @@ static void rebuild_update_schema(void){
   /* Add the user.mtime column if it is missing. (2011-04-27)
   */
   if( !db_table_has_column("repository", "user", "mtime") ){
+    db_unprotect(PROTECT_ALL);
     db_multi_exec(
       "CREATE TEMP TABLE temp_user AS SELECT * FROM user;"
       "DROP TABLE user;"
@@ -74,15 +75,18 @@ static void rebuild_update_schema(void){
                " ipaddr, cexpire, info, now(), photo FROM temp_user;"
       "DROP TABLE temp_user;"
     );
+    db_protect_pop();
   }
 
   /* Add the config.mtime column if it is missing.  (2011-04-27)
   */
   if( !db_table_has_column("repository", "config", "mtime") ){
+    db_unprotect(PROTECT_CONFIG);
     db_multi_exec(
       "ALTER TABLE config ADD COLUMN mtime INTEGER;"
       "UPDATE config SET mtime=now();"
     );
+    db_protect_pop();
   }
 
   /* Add the shun.mtime and shun.scom columns if they are missing.
@@ -381,9 +385,10 @@ int rebuild_db(int randomize, int doOut, int doClustering){
   if (ttyOutput && !g.fQuiet) {
     percent_complete(0);
   }
-  alert_triggers_disable();
+  manifest_disable_event_triggers();
   rebuild_update_schema();
   blob_init(&sql, 0, 0);
+  db_unprotect(PROTECT_ALL);
   db_prepare(&q,
      "SELECT name FROM sqlite_schema /*scan*/"
      " WHERE type='table'"
@@ -391,7 +396,7 @@ int rebuild_db(int randomize, int doOut, int doClustering){
                        "'config','shun','private','reportfmt',"
                        "'concealed','accesslog','modreq',"
                        "'purgeevent','purgeitem','unversioned',"
-                       "'subscriber','pending_alert','alert_bounce')"
+                       "'subscriber','pending_alert','alert_bounce','chat')"
      " AND name NOT GLOB 'sqlite_*'"
      " AND name NOT GLOB 'fx_*'"
   );
@@ -472,11 +477,11 @@ int rebuild_db(int randomize, int doOut, int doClustering){
     processCnt += incrSize;
     percent_complete((processCnt*1000)/totalSize);
   }
-  alert_triggers_enable();
   if(!g.fQuiet && ttyOutput ){
     percent_complete(1000);
     fossil_print("\n");
   }
+  db_protect_pop();
   return errCnt;
 }
 
@@ -669,6 +674,7 @@ void rebuild_database(void){
   verify_all_options();
 
   db_begin_transaction();
+  db_unprotect(PROTECT_ALL);
   if( !compressOnlyFlag ){
     search_drop_index();
     ttyOutput = 1;
@@ -722,6 +728,7 @@ void rebuild_database(void){
     }
   }
   if( runReindex ) search_rebuild_index();
+  db_protect_pop();
   if( showStats ){
     static const struct { int idx; const char *zLabel; } aStat[] = {
        { CFTYPE_ANY,       "Artifacts:" },
@@ -757,6 +764,7 @@ void rebuild_database(void){
 void test_detach_cmd(void){
   db_find_and_open_repository(0, 2);
   db_begin_transaction();
+  db_unprotect(PROTECT_CONFIG);
   db_multi_exec(
     "DELETE FROM config WHERE name GLOB 'last-sync-*';"
     "DELETE FROM config WHERE name GLOB 'sync-*:*';"
@@ -765,6 +773,7 @@ void test_detach_cmd(void){
     "UPDATE config SET value='detached-' || value"
     " WHERE name='project-name' AND value NOT GLOB 'detached-*';"
   );
+  db_protect_pop();
   db_end_transaction(0);
 }
 
@@ -912,6 +921,7 @@ void scrub_cmd(void){
     delete_private_content();
   }
   if( !privateOnly ){
+    db_unprotect(PROTECT_ALL);
     db_multi_exec(
       "UPDATE user SET pw='';"
       "DELETE FROM config WHERE name GLOB 'last-sync-*';"
@@ -933,12 +943,16 @@ void scrub_cmd(void){
         "DROP TABLE IF EXISTS purgeitem;\n"
         "DROP TABLE IF EXISTS admin_log;\n"
         "DROP TABLE IF EXISTS vcache;\n"
+        "DROP TABLE IF EXISTS chat;\n"
       );
     }
+    db_protect_pop();
   }
   if( !bNeedRebuild ){
     db_end_transaction(0);
+    db_unprotect(PROTECT_ALL);
     db_multi_exec("VACUUM;");
+    db_protect_pop();
   }else{
     rebuild_db(0, 1, 0);
     db_end_transaction(0);
