@@ -18,6 +18,9 @@
 ** Setup pages associated with user management.  The code in this
 ** file was formerly part of the "setup.c" module, but has been broken
 ** out into its own module to improve maintainability.
+**
+** Note:  Do not confuse "Users" with "Subscribers".  Code to deal with
+** subscribers is over in the "alerts.c" source file.
 */
 #include "config.h"
 #include <assert.h>
@@ -51,6 +54,7 @@ void setup_ulist(void){
   if( alert_tables_exist() ){
     style_submenu_element("Subscribers", "subscribers");
   }
+  style_set_current_feature("setup");
   style_header("User List");
   if( (zWith==0 || zWith[0]==0) && !bUnusedOnly ){
     @ <table border=1 cellpadding=2 cellspacing=0 class='userTable'>
@@ -111,9 +115,10 @@ void setup_ulist(void){
     style_submenu_element("Unused", "setup_ulist?unused");
   }
   @ <table border=1 cellpadding=2 cellspacing=0 class='userTable sortable' \
-  @  data-column-types='ktxTTK' data-init-sort='2'>
+  @  data-column-types='ktxTTKt' data-init-sort='2'>
   @ <thead><tr>
-  @ <th>Login Name<th>Caps<th>Info<th>Date<th>Expire<th>Last Login</tr></thead>
+  @ <th>Login Name<th>Caps<th>Info<th>Date<th>Expire<th>Last Login\
+  @ <th>Alerts</tr></thead>
   @ <tbody>
   db_multi_exec(
     "CREATE TEMP TABLE lastAccess(uname TEXT PRIMARY KEY, atime REAL)"
@@ -130,6 +135,12 @@ void setup_ulist(void){
       " GROUP BY 1;"
     );
   }
+  if( !db_table_exists("repository","subscriber") ){
+    db_multi_exec(
+      "CREATE TEMP TABLE subscriber(suname PRIMARY KEY, ssub, subscriberId)"
+      "WITHOUT ROWID;"
+    );
+  }
   if( bUnusedOnly ){
     zWith = mprintf(
         " AND login NOT IN ("
@@ -144,13 +155,15 @@ void setup_ulist(void){
     zWith = "";
   }
   db_prepare(&s,
-     "SELECT uid, login, cap, info, date(mtime,'unixepoch'),"
+     "SELECT uid, login, cap, info, date(user.mtime,'unixepoch'),"
      "       lower(login) AS sortkey, "
      "       CASE WHEN info LIKE '%%expires 20%%'"
              "    THEN substr(info,instr(lower(info),'expires')+8,10)"
              "    END AS exp,"
-             "atime"
+             "atime,"
+     "       subscriber.ssub, subscriber.subscriberId"
      "  FROM user LEFT JOIN lastAccess ON login=uname"
+     "            LEFT JOIN subscriber ON login=suname"
      " WHERE login NOT IN ('anonymous','nobody','developer','reader') %s"
      " ORDER BY sortkey", zWith/*safe-for-%s*/
   );
@@ -165,6 +178,8 @@ void setup_ulist(void){
     const char *zExp = db_column_text(&s,6);
     double rATime = db_column_double(&s,7);
     char *zAge = 0;
+    const char *zSub;
+    int sid = db_column_int(&s,9);
     if( rATime>0.0 ){
       zAge = human_readable_age(rNow - rATime);
     }
@@ -176,13 +191,21 @@ void setup_ulist(void){
     @ <td>%h(zDate?zDate:"")
     @ <td>%h(zExp?zExp:"")
     @ <td data-sortkey='%f(rATime)' style='white-space:nowrap'>%s(zAge?zAge:"")
+    if( db_column_type(&s,8)==SQLITE_NULL ){
+      @ <td>
+    }else if( (zSub = db_column_text(&s,8))==0 || zSub[0]==0 ){
+      @ <td><a href="%R/alerts?sid=%d(sid)"><i>off</i></a>
+    }else{
+      @ <td><a href="%R/alerts?sid=%d(sid)">%h(zSub)</a>
+    }
+
     @ </tr>
     fossil_free(zAge);
   }
   @ </tbody></table>
   db_finalize(&s);
   style_table_sorter();
-  style_footer();
+  style_finish_page();
 }
 
 /*
@@ -193,6 +216,7 @@ void setup_ulist(void){
 ** factored out for improved presentation.
 */
 void setup_ulist_notes(void){
+  style_set_current_feature("setup");
   style_header("User Configuration Notes");
   @ <h1>User Configuration Notes:</h1>
   @ <ol>
@@ -228,7 +252,7 @@ void setup_ulist_notes(void){
   capabilities_table(CAPCLASS_ALL);
   @ </li>
   @ </ol>
-  style_footer();
+  style_finish_page();
 }
 
 /*
@@ -238,6 +262,7 @@ void setup_ulist_notes(void){
 ** code letters.
 */
 void setup_ucap_list(void){
+  style_set_current_feature("setup");
   style_header("User Capability Codes");
   @ <h1>All capabilities</h1>
   capabilities_table(CAPCLASS_ALL);
@@ -255,7 +280,7 @@ void setup_ucap_list(void){
   capabilities_table(CAPCLASS_SUPER);
   @ <h1>Miscellaneous capabilities</h1>
   capabilities_table(CAPCLASS_OTHER);
-  style_footer();
+  style_finish_page();
 }
 
 /*
@@ -317,7 +342,14 @@ void user_edit(void){
     int n;
     if( P("verifydelete") ){
       /* Verified delete user request */
+      db_unprotect(PROTECT_USER);
+      if( db_table_exists("repository","subscriber") ){
+        /* Also delete any subscriptions associated with this user */
+        db_multi_exec("DELETE FROM subscriber WHERE suname="
+                      "(SELECT login FROM user WHERE uid=%d)", uid);
+      }
       db_multi_exec("DELETE FROM user WHERE uid=%d", uid);
+      db_protect_pop();
       moderation_disapprove_for_missing_users();
       admin_log("Deleted user [%s] (uid %d).",
                 PD("login","???")/*safe-for-%s*/, uid);
@@ -337,6 +369,8 @@ void user_edit(void){
     }
   }
 
+  style_set_current_feature("setup");
+
   /* If we have all the necessary information, write the new or
   ** modified user record.  After writing the user record, redirect
   ** to the page that displays a list of users.
@@ -382,7 +416,7 @@ void user_edit(void){
       @
       @ <p><a href="setup_uedit?id=%d(uid)&referer=%T(zRef)">
       @ [Bummer]</a></p>
-      style_footer();
+      style_finish_page();
       return;
     }
     if( isValidPwString(zPw) ){
@@ -399,15 +433,17 @@ void user_edit(void){
       @
       @ <p><a href="setup_uedit?id=%d(uid)&referer=%T(zRef)">
       @ [Bummer]</a></p>
-      style_footer();
+      style_finish_page();
       return;
     }
     login_verify_csrf_secret();
+    db_unprotect(PROTECT_USER);
     db_multi_exec(
        "REPLACE INTO user(uid,login,info,pw,cap,mtime) "
        "VALUES(nullif(%d,0),%Q,%Q,%Q,%Q,now())",
       uid, zLogin, P("info"), zPw, zCap
     );
+    db_protect_pop();
     setup_incr_cfgcnt();
     admin_log( "Updated user [%q] with capabilities [%q].",
                zLogin, zCap );
@@ -434,7 +470,9 @@ void user_edit(void){
         zLogin, P("pw"), zLogin, P("info"), zCap,
         zOldLogin
       );
+      db_unprotect(PROTECT_USER);
       login_group_sql(blob_str(&sql), "<li> ", " </li>\n", &zErr);
+      db_protect_pop();
       blob_reset(&sql);
       admin_log( "Updated user [%q] in all login groups "
                  "with capabilities [%q].",
@@ -447,7 +485,7 @@ void user_edit(void){
         @
         @ <p><a href="setup_uedit?id=%d(uid)&referer=%T(zRef)">
         @ [Bummer]</a></p>
-        style_footer();
+        style_finish_page();
         return;
       }
     }
@@ -645,6 +683,8 @@ void user_edit(void){
   @  Email Alerts%s(B('7'))</label>
   @  <li><label><input type="checkbox" name="aA"%s(oa['A']) />
   @  Send Announcements%s(B('A'))</label>
+  @  <li><label><input type="checkbox" name="aC"%s(oa['C']) />
+  @  Chatroom%s(B('C'))</label>
   @  <li><label><input type="checkbox" name="aD"%s(oa['D']) />
   @  Enable Debug%s(B('D'))</label>
   @ </ul></div>
@@ -666,8 +706,9 @@ void user_edit(void){
       @   name="pw" value="**********" /></td>
     }else{
       /* Show an empty password as an empty input field */
+      char *zRPW = fossil_random_password(12);
       @   <td><input aria-labelledby="supw" type="password" name="pw" \
-      @        autocomplete="off" value="" /></td>
+      @   autocomplete="off" value="" /> Password suggestion: %z(zRPW)</td>
     }
     @ </tr>
   }
@@ -865,5 +906,5 @@ void user_edit(void){
   @ but less than a <span class="usertype">developer</span>.
   @ </p></li>
   @ </ul>
-  style_footer();
+  style_finish_page();
 }
