@@ -87,10 +87,13 @@ static struct SkinDetail {
   const char *zName;      /* Name of the detail */
   const char *zValue;     /* Value of the detail */
 } aSkinDetail[] = {
-  { "timeline-arrowheads",        "1"  },
-  { "timeline-circle-nodes",      "0"  },
-  { "timeline-color-graph-lines", "0"  },
-  { "white-foreground",           "0"  },
+  { "pikchr-fontscale",           ""      },
+  { "pikchr-foreground",          ""      },
+  { "pikchr-scale",               ""      },
+  { "timeline-arrowheads",        "1"     },
+  { "timeline-circle-nodes",      "0"     },
+  { "timeline-color-graph-lines", "0"     },
+  { "white-foreground",           "0"     },
 };
 
 /*
@@ -362,10 +365,12 @@ static char *getSkin(const char *zName){
         fossil_free(zLabel);
       }
     }
+    db_unprotect(PROTECT_CONFIG);
     blob_appendf(&val,
        "REPLACE INTO config(name,value,mtime) VALUES(%Q,%Q,now());\n",
        azSkinFile[i], z
     );
+    db_protect_pop();
   }
   return blob_str(&val);
 }
@@ -384,12 +389,13 @@ static int skinRename(void){
   if( zOldName==0 ) return 0;
   if( zNewName==0 || zNewName[0]==0 || (ex = skinExists(zNewName))!=0 ){
     if( zNewName==0 ) zNewName = zOldName;
+    style_set_current_feature("skins");
     style_header("Rename A Skin");
     if( ex ){
       @ <p><span class="generalError">There is already another skin
       @ named "%h(zNewName)".  Choose a different name.</span></p>
     }
-    @ <form action="%s(g.zTop)/setup_skin_admin" method="post"><div>
+    @ <form action="%R/setup_skin_admin" method="post"><div>
     @ <table border="0"><tr>
     @ <tr><td align="right">Current name:<td align="left"><b>%h(zOldName)</b>
     @ <tr><td align="right">New name:<td align="left">
@@ -401,13 +407,15 @@ static int skinRename(void){
     @ </table>
     login_insert_csrf_secret();
     @ </div></form>
-    style_footer();
+    style_finish_page();
     return 1;
   }
+  db_unprotect(PROTECT_CONFIG);
   db_multi_exec(
     "UPDATE config SET name='skin:%q' WHERE name='skin:%q';",
     zNewName, zOldName
   );
+  db_protect_pop();
   return 0;
 }
 
@@ -424,12 +432,13 @@ static int skinSave(const char *zCurrent){
   }
   if( zNewName==0 || zNewName[0]==0 || (ex = skinExists(zNewName))!=0 ){
     if( zNewName==0 ) zNewName = "";
+    style_set_current_feature("skins");
     style_header("Save Current Skin");
     if( ex ){
       @ <p><span class="generalError">There is already another skin
       @ named "%h(zNewName)".  Choose a different name.</span></p>
     }
-    @ <form action="%s(g.zTop)/setup_skin_admin" method="post"><div>
+    @ <form action="%R/setup_skin_admin" method="post"><div>
     @ <table border="0"><tr>
     @ <tr><td align="right">Name for this skin:<td align="left">
     @ <input type="text" size="35" name="svname" value="%h(zNewName)">
@@ -439,14 +448,16 @@ static int skinSave(const char *zCurrent){
     @ </table>
     login_insert_csrf_secret();
     @ </div></form>
-    style_footer();
+    style_finish_page();
     return 1;
   }
+  db_unprotect(PROTECT_CONFIG);
   db_multi_exec(
     "INSERT OR IGNORE INTO config(name, value, mtime)"
     "VALUES('skin:%q',%Q,now())",
     zNewName, zCurrent
   );
+  db_protect_pop();
   return 0;
 }
 
@@ -476,11 +487,13 @@ void setup_skin_admin(void){
     aBuiltinSkin[i].zSQL = getSkin(aBuiltinSkin[i].zLabel);
   }
 
+  style_set_current_feature("skins");
+
   if( cgi_csrf_safe(1) ){
     /* Process requests to delete a user-defined skin */
     if( P("del1") && (zName = skinVarName(P("sn"), 1))!=0 ){
       style_header("Confirm Custom Skin Delete");
-      @ <form action="%s(g.zTop)/setup_skin_admin" method="post"><div>
+      @ <form action="%R/setup_skin_admin" method="post"><div>
       @ <p>Deletion of a custom skin is a permanent action that cannot
       @ be undone.  Please confirm that this is what you want to do:</p>
       @ <input type="hidden" name="sn" value="%h(P("sn"))" />
@@ -488,17 +501,21 @@ void setup_skin_admin(void){
       @ <input type="submit" name="cancel" value="Cancel - Do Not Delete" />
       login_insert_csrf_secret();
       @ </div></form>
-      style_footer();
+      style_finish_page();
       db_end_transaction(1);
       return;
     }
     if( P("del2")!=0 && (zName = skinVarName(P("sn"), 1))!=0 ){
+      db_unprotect(PROTECT_CONFIG);
       db_multi_exec("DELETE FROM config WHERE name=%Q", zName);
+      db_protect_pop();
     }
     if( P("draftdel")!=0 ){
       const char *zDraft = P("name");
       if( sqlite3_strglob("draft[1-9]",zDraft)==0 ){
+        db_unprotect(PROTECT_CONFIG);
         db_multi_exec("DELETE FROM config WHERE name GLOB '%q-*'", zDraft);
+        db_protect_pop();
       }
     }
     if( skinRename() || skinSave(zCurrent) ){
@@ -523,11 +540,13 @@ void setup_skin_admin(void){
         seen = db_exists("SELECT 1 FROM config WHERE name GLOB 'skin:*'"
                          " AND value=%Q", zCurrent);
         if( !seen ){
+          db_unprotect(PROTECT_CONFIG);
           db_multi_exec(
             "INSERT INTO config(name,value,mtime) VALUES("
             "  strftime('skin:Backup On %%Y-%%m-%%d %%H:%%M:%%S'),"
             "  %Q,now())", zCurrent
           );
+          db_protect_pop();
         }
       }
       seen = 0;
@@ -535,14 +554,18 @@ void setup_skin_admin(void){
         if( fossil_strcmp(aBuiltinSkin[i].zDesc, z)==0 ){
           seen = 1;
           zCurrent = aBuiltinSkin[i].zSQL;
+          db_unprotect(PROTECT_CONFIG);
           db_multi_exec("%s", zCurrent/*safe-for-%s*/);
+          db_protect_pop();
           break;
         }
       }
       if( !seen ){
         zName = skinVarName(z,0);
         zCurrent = db_get(zName, 0);
+        db_unprotect(PROTECT_CONFIG);
         db_multi_exec("%s", zCurrent/*safe-for-%s*/);
+        db_protect_pop();
       }
     }
   }
@@ -560,7 +583,7 @@ void setup_skin_admin(void){
       @ (Currently In Use)
       seenCurrent = 1;
     }else{
-      @ <form action="%s(g.zTop)/setup_skin_admin" method="post">
+      @ <form action="%R/setup_skin_admin" method="post">
       @ <input type="hidden" name="sn" value="%h(z)" />
       @ <input type="submit" name="load" value="Install" />
       if( pAltSkin==&aBuiltinSkin[i] ){
@@ -586,7 +609,7 @@ void setup_skin_admin(void){
       @ in the CONFIG table:</h2></td></tr>
     }
     @ <tr><td>%d(i).<td>%h(zN)<td>&nbsp;&nbsp;<td>
-    @ <form action="%s(g.zTop)/setup_skin_admin" method="post">
+    @ <form action="%R/setup_skin_admin" method="post">
     if( fossil_strcmp(zV, zCurrent)==0 ){
       @ (Currently In Use)
       seenCurrent = 1;
@@ -604,7 +627,7 @@ void setup_skin_admin(void){
     @ <tr><td colspan=4><h2>Current skin in css/header/footer/details entries \
     @ in the CONFIG table:</h2></td></tr>
     @ <tr><td>%d(i).<td><i>Current</i><td>&nbsp;&nbsp;<td>
-    @ <form action="%s(g.zTop)/setup_skin_admin" method="post">
+    @ <form action="%R/setup_skin_admin" method="post">
     @ <input type="submit" name="save" value="Backup">
     @ </form>
   }
@@ -623,7 +646,7 @@ void setup_skin_admin(void){
       @ in the CONFIG table:</h2></td></tr>
     }
     @ <tr><td>%d(i).<td>%h(zN)<td>&nbsp;&nbsp;<td>
-    @ <form action="%s(g.zTop)/setup_skin_admin" method="post">
+    @ <form action="%R/setup_skin_admin" method="post">
     @ <input type="submit" name="draftdel" value="Delete">
     @ <input type="hidden" name="name" value="%h(zN)">
     @ </form></tr>
@@ -631,7 +654,7 @@ void setup_skin_admin(void){
   db_finalize(&q);
 
   @ </table>
-  style_footer();
+  style_finish_page();
   db_end_transaction(0);
 }
 
@@ -679,7 +702,7 @@ static void skin_emit_skin_selector(
 static const char *skin_file_content(const char *zLabel, const char *zFile){
   const char *zResult;
   if( fossil_strcmp(zLabel, "current")==0 ){
-    zResult = db_get(zFile, "");
+    zResult = skin_get(zFile);
   }else if( sqlite3_strglob("draft[1-9]", zLabel)==0 ){
     zResult = db_get_mprintf("", "%s-%s", zLabel, zFile);
   }else{
@@ -779,12 +802,13 @@ void setup_skinedit(void){
   }
 
   db_begin_transaction();
+  style_set_current_feature("skins");
   style_header("%s", zTitle);
   for(j=0; j<count(aSkinAttr); j++){
     style_submenu_element(aSkinAttr[j].zSubmenu,
           "%R/setup_skinedit?w=%d&basis=%h&sk=%d",j,zBasis,iSkin);
   }
-  @ <form action="%s(g.zTop)/setup_skinedit" method="post"><div>
+  @ <form action="%R/setup_skinedit" method="post"><div>
   login_insert_csrf_secret();
   @ <input type='hidden' name='w' value='%d(ii)'>
   @ <input type='hidden' name='sk' value='%d(iSkin)'>
@@ -828,7 +852,7 @@ void setup_skinedit(void){
     blob_reset(&out);
   }
   @ </div></form>
-  style_footer();
+  style_finish_page();
   db_end_transaction(0);
 }
 
@@ -869,11 +893,13 @@ static void skin_publish(int iSkin){
                        " AND value=%Q", zCurrent);
   }
   if( !seen ){
+    db_unprotect(PROTECT_CONFIG);
     db_multi_exec(
       "INSERT INTO config(name,value,mtime) VALUES("
       "  strftime('skin:Backup On %%Y-%%m-%%d %%H:%%M:%%S'),"
       "  %Q,now())", zCurrent
     );
+    db_protect_pop();
   }
 
   /* Publish draft iSkin */
@@ -943,6 +969,7 @@ void setup_skin(void){
     skin_publish(iSkin);
   }
 
+  style_set_current_feature("skins");
   style_header("Customize Skin");
 
   @ <p>Customize the look of this Fossil repository by making changes
@@ -1104,5 +1131,5 @@ void setup_skin(void){
     @ for cleanup and recovery actions.
   }
   builtin_request_js("skin.js");
-  style_footer();
+  style_finish_page();
 }
