@@ -771,6 +771,7 @@ void setup_smtp(void){
     return;
   }
   db_begin_transaction();
+  style_set_current_feature("smtp");
   style_header("Email Server Setup");
   if( db_table_exists("repository","emailroute") ){
     style_submenu_element("emailblob table", "%R/emailblob");
@@ -807,7 +808,7 @@ void setup_smtp(void){
   @    &larr; Add a new email address
   @   </form>
   @ </table>
-  style_footer();
+  style_finish_page();
   db_end_transaction(0);
 }
 
@@ -835,6 +836,7 @@ void setup_smtp_route(void){
     login_needed(0);
     return;
   }
+  style_set_current_feature("smtp");
   style_header("Email Route Editor");
 
   if( P("edit") && cgi_csrf_safe(1) && zEAddr!=0 && zEPolicy!=0 ){
@@ -926,7 +928,7 @@ smtp_route_edit:
   @
   @ <p>To delete a route &rarr; erase all text from the "Routing" field then
   @ press the "Apply" button.
-  style_footer();
+  style_finish_page();
 }
 
 #if LOCAL_INTERFACE
