@@ -35,7 +35,7 @@
 **      style_submenu_multichoice()
 **      style_submenu_sql()
 **
-** prior to calling style_footer().  The style_footer() routine
+** prior to calling style_finish_page().  The style_finish_page() routine
 ** will generate the appropriate HTML text just below the main
 ** menu.
 */
@@ -86,6 +86,12 @@ static unsigned adUnitFlags = 0;
 */
 static int submenuEnable = 1;
 
+/*
+** Disable content-security-policy.
+** Warning:  Do not disable the CSP without careful consideration!
+*/
+static int disableCSP = 0;
+
 /*
 ** Flags for various javascript files needed prior to </body>
 */
@@ -142,6 +148,7 @@ static Blob blobOnLoad = BLOB_INITIALIZER;
 char *xhref(const char *zExtra, const char *zFormat, ...){
   char *zUrl;
   va_list ap;
+  if( !g.perm.Hyperlink ) return fossil_strdup("");
   va_start(ap, zFormat);
   zUrl = vmprintf(zFormat, ap);
   va_end(ap);
@@ -166,6 +173,7 @@ char *xhref(const char *zExtra, const char *zFormat, ...){
 char *chref(const char *zExtra, const char *zFormat, ...){
   char *zUrl;
   va_list ap;
+  if( !g.perm.Hyperlink ) return fossil_strdup("");
   va_start(ap, zFormat);
   zUrl = vmprintf(zFormat, ap);
   va_end(ap);
@@ -181,6 +189,7 @@ char *chref(const char *zExtra, const char *zFormat, ...){
 char *href(const char *zFormat, ...){
   char *zUrl;
   va_list ap;
+  if( !g.perm.Hyperlink ) return fossil_strdup("");
   va_start(ap, zFormat);
   zUrl = vmprintf(zFormat, ap);
   va_end(ap);
@@ -524,10 +533,13 @@ char *style_csp(int toHeader){
    "default-src 'self' data:; "
    "script-src 'self' 'nonce-$nonce'; "
    "style-src 'self' 'unsafe-inline'";
-  const char *zFormat = db_get("default-csp","");
+  const char *zFormat;
   Blob csp;
   char *zNonce;
   char *zCsp;
+  int i;
+  if( disableCSP ) return fossil_strdup("");
+  zFormat = db_get("default-csp","");
   if( zFormat[0]==0 ){
     zFormat = zBackupCSP;
   }
@@ -539,21 +551,36 @@ char *style_csp(int toHeader){
   }
   blob_append(&csp, zFormat, -1);
   zCsp = blob_str(&csp);
+  /* No whitespace other than actual space characters allowed in the CSP
+  ** string.  See https://fossil-scm.org/forum/forumpost/d29e3af43c */
+  for(i=0; zCsp[i]; i++){ if( fossil_isspace(zCsp[i]) ) zCsp[i] = ' '; }
   if( toHeader ){
     cgi_printf_header("Content-Security-Policy: %s\r\n", zCsp);
   }
   return zCsp;
 }
 
+/*
+** Disable content security policy for the current page.
+** WARNING:  Do not do this lightly!
+**
+** This routine must be called before the CSP is sued by 
+** style_header().
+*/
+void style_disable_csp(void){
+  disableCSP = 1;
+}
+
 /*
 ** Default HTML page header text through <body>.  If the repository-specific
 ** header template lacks a <body> tag, then all of the following is
 ** prepended.
 */
-static char zDfltHeader[] = 
+static const char zDfltHeader[] = 
 @ <html>
 @ <head>
 @ <base href="$baseurl/$current_page" />
+@ <meta charset="UTF-8">
 @ <meta http-equiv="Content-Security-Policy" content="$default_csp" />
 @ <meta name="viewport" content="width=device-width, initial-scale=1.0">
 @ <title>$<project_name>: $<title></title>
@@ -561,9 +588,49 @@ static char zDfltHeader[] =
 @  href="$home/timeline.rss" />
 @ <link rel="stylesheet" href="$stylesheet_url" type="text/css" />
 @ </head>
-@ <body>
+@ <body class="$current_feature">
 ;
 
+/*
+** Returns the default page header.
+*/
+const char *get_default_header(){
+  return zDfltHeader;
+}
+
+/*
+** Given a URL path, extract the first element as a "feature" name,
+** used as the <body class="FEATURE"> value by default, though
+** later-running code may override this, typically to group multiple
+** Fossil UI URLs into a single "feature" so you can have per-feature
+** CSS rules.
+**
+** For example, "body.forum div.markdown blockquote" targets only
+** block quotes made in forum posts, leaving other Markdown quotes
+** alone.  Because feature class "forum" groups /forummain, /forumpost,
+** and /forume2, it works across all renderings of Markdown to HTML
+** within the Fossil forum feature.
+*/
+static const char* feature_from_page_path(const char *zPath){
+  const char* zSlash = strchr(zPath, '/');
+  if (zSlash) {
+    return fossil_strndup(zPath, zSlash - zPath);
+  } else {
+    return zPath;
+  }
+}
+
+/*
+** Override the value of the TH1 variable current_feature, its default
+** set by feature_from_page_path().  We do not call this from
+** style_init_th1_vars() because that uses Th_MaybeStore() instead to
+** allow webpage implementations to call this before style_header()
+** to override that "maybe" default with something better.
+*/
+void style_set_current_feature(const char* zFeature){
+  Th_Store("current_feature", zFeature);
+}
+
 /*
 ** Initialize all the default TH1 variables
 */
@@ -600,6 +667,7 @@ static void style_init_th1_vars(const char *zTitle){
   if( !login_is_nobody() ){
     Th_Store("login", g.zLogin);
   }
+  Th_MaybeStore("current_feature", feature_from_page_path(local_zCurrentPage) );
 }
 
 /*
@@ -699,18 +767,18 @@ void style_table_sorter(void){
 ** Generate code to load all required javascript files.
 */
 static void style_load_all_js_files(void){
-  if( needHrefJs ){
+  if( needHrefJs && g.perm.Hyperlink ){
     int nDelay = db_get_int("auto-hyperlink-delay",0);
     int bMouseover = db_get_boolean("auto-hyperlink-mouseover",0);
     @ <script id='href-data' type='application/json'>\
     @ {"delay":%d(nDelay),"mouseover":%d(bMouseover)}</script>
   }
-  @ <script nonce="%h(style_nonce())">
+  @ <script nonce="%h(style_nonce())">/* style.c:%d(__LINE__) */
   @ function debugMsg(msg){
   @ var n = document.getElementById("debugMsg");
   @ if(n){n.textContent=msg;}
   @ }
-  if( needHrefJs ){
+  if( needHrefJs && g.perm.Hyperlink ){
     @ /* href.js */
     cgi_append_content(builtin_text("href.js"),-1);
   }
@@ -724,9 +792,20 @@ static void style_load_all_js_files(void){
 }
 
 /*
-** Draw the footer at the bottom of the page.
+** Invoke this routine after all of the content for a webpage has been
+** generated.  This routine should be called once for every webpage, at
+** or near the end of page generation.  This routine does the following:
+**
+**   *  Populates the header of the page, including setting up appropriate
+**      submenu elements.  The header generation is deferred until this point
+**      so that we know that all style_submenu_element() and similar have
+**      been received.
+**
+**   *  Finalizes the page content.
+**
+**   *  Appends the footer.
 */
-void style_footer(void){
+void style_finish_page(){
   const char *zFooter;
   const char *zAd = 0;
   unsigned int mAdFlags = 0;
@@ -852,27 +931,21 @@ void style_footer(void){
     @ <div class="adunit_right">
     cgi_append_content(zAd, -1);
     @ </div>
-  }else{
-    if( zAd ){
-      @ <div class="adunit_banner">
-      cgi_append_content(zAd, -1);
-      @ </div>
-    }
-    @ <div class="content"><span id="debugMsg"></span>
+  }else if( zAd ){
+    @ <div class="adunit_banner">
+    cgi_append_content(zAd, -1);
+    @ </div>
   }
+
+  @ <div class="content"><span id="debugMsg"></span>
   cgi_destination(CGI_BODY);
 
   if( sideboxUsed ){
-    /* Put the footer at the bottom of the page.
-    ** the additional clear/both is needed to extend the content
-    ** part to the end of an optional sidebox.
-    */
     @ <div class="endContent"></div>
   }
   @ </div>
 
-
-
+  /* Put the footer at the bottom of the page. */
   zFooter = skin_get("footer");
   if( sqlite3_strlike("%</body>%", zFooter, 0)==0 ){
     style_load_all_js_files();
@@ -1129,6 +1202,7 @@ void webpage_error(const char *zFormat, ...){
     isAuth = 1;
   }
   cgi_load_environment();
+  style_set_current_feature(zFormat[0]==0 ? "test" : "error");
   if( zFormat[0] ){
     va_list ap;
     va_start(ap, zFormat);
@@ -1171,6 +1245,9 @@ void webpage_error(const char *zFormat, ...){
     }
     @ g.zRepositoryName = %h(g.zRepositoryName)<br />
     @ load_average() = %f(load_average())<br />
+#ifndef _WIN32
+    @ RSS = %.2f(fossil_rss()/1000000.0) MB</br />
+#endif
     @ cgi_csrf_safe(0) = %d(cgi_csrf_safe(0))<br />
     @ fossil_exe_id() = %h(fossil_exe_id())<br />
     @ <hr />
@@ -1183,10 +1260,12 @@ void webpage_error(const char *zFormat, ...){
       @ </pre>
     }
   }
-  style_footer();
-  if( zErr ){
+  if( zErr && zErr[0] ){
+    style_finish_page();
     cgi_reply();
     fossil_exit(1);
+  }else{
+    style_finish_page();
   }
 }
 
@@ -1206,6 +1285,26 @@ void webpage_assert_page(const char *zFile, int iLine, const char *zExpr){
   webpage_error("assertion fault at %s:%d - %s", zFile, iLine, zExpr);
 }
 
+/*
+** Issue a 404 Not Found error for a webpage
+*/
+void webpage_notfound_error(const char *zFormat, ...){
+  char *zMsg;
+  va_list ap;
+  if( zFormat ){
+    va_start(ap, zFormat);
+    zMsg = vmprintf(zFormat, ap);
+    va_end(ap);
+  }else{
+    zMsg = "Not Found";
+  }
+  style_set_current_feature("enotfound");
+  style_header("Not Found");
+  @ <p>%h(zMsg)</p>
+  cgi_set_status(404, "Not Found");
+  style_finish_page();
+}
+
 #if INTERFACE
 # define webpage_assert(T) if(!(T)){webpage_assert_page(__FILE__,__LINE__,#T);}
 #endif
@@ -1232,12 +1331,12 @@ static char * style_next_input_id(){
 **
 ** Resulting structure:
 **
-** <span class='input-with-label' title={{zTip}} id={{zWrapperId}}>
+** <div class='input-with-label' title={{zTip}} id={{zWrapperId}}>
 **   <input type='checkbox' name={{zFieldName}} value={{zValue}}
 **          id='A RANDOM VALUE'
 **          {{isChecked ? " checked : ""}}/>
 **   <label for='ID OF THE INPUT FIELD'>{{zLabel}}</label>
-** </span>
+** </div>
 **
 ** zLabel, and zValue are required. zFieldName, zWrapperId, and zTip
 ** are may be NULL or empty.
@@ -1251,7 +1350,7 @@ void style_labeled_checkbox(const char * zWrapperId,
                             const char * zValue, int isChecked,
                             const char * zTip){
   char * zLabelID = style_next_input_id();
-  CX("<span class='input-with-label'");
+  CX("<div class='input-with-label'");
   if(zTip && *zTip){
     CX(" title='%h'", zTip);
   }
@@ -1264,7 +1363,7 @@ void style_labeled_checkbox(const char * zWrapperId,
   }
   CX("value='%T'%s/>",
      zValue ? zValue : "", isChecked ? " checked" : "");
-  CX("<label for='%s'>%h</label></span>", zLabelID, zLabel);
+  CX("<label for='%s'>%h</label></div>", zLabelID, zLabel);
   fossil_free(zLabelID);
 }
 
@@ -1298,10 +1397,10 @@ void style_labeled_checkbox(const char * zWrapperId,
 **
 ** The structure of the emitted HTML is:
 **
-** <span class='input-with-label' title={{zToolTip}} id={{zWrapperId}}>
+** <div class='input-with-label' title={{zToolTip}} id={{zWrapperId}}>
 **   <label for='SELECT ELEMENT ID'>{{zLabel}}</label>
 **   <select id='RANDOM ID' name={{zFieldName}}>...</select>
-** </span>
+** </div>
 **
 ** Example:
 **
@@ -1320,7 +1419,7 @@ void style_select_list_int(const char * zWrapperId,
   va_list vargs;
 
   va_start(vargs,selectedVal);
-  CX("<span class='input-with-label'");
+  CX("<div class='input-with-label'");
   if(zToolTip && *zToolTip){
     CX(" title='%h'",zToolTip);
   }
@@ -1349,7 +1448,7 @@ void style_select_list_int(const char * zWrapperId,
     CX("</option>\n");
   }
   CX("</select>\n");
-  CX("</span>\n");
+  CX("</div>\n");
   va_end(vargs);
   fossil_free(zLabelID);
 }
@@ -1384,7 +1483,7 @@ void style_select_list_str(const char * zWrapperId,
   if(!zSelectedVal){
     zSelectedVal = __FILE__/*some string we'll never match*/;
   }
-  CX("<span class='input-with-label'");
+  CX("<div class='input-with-label'");
   if(zToolTip && *zToolTip){
     CX(" title='%h'",zToolTip);
   }
@@ -1413,152 +1512,41 @@ void style_select_list_str(const char * zWrapperId,
     CX("</option>\n");
   }
   CX("</select>\n");
-  CX("</span>\n");
+  CX("</div>\n");
   va_end(vargs);
   fossil_free(zLabelID);
 }
 
-
 /*
-** The first time this is called, it emits code to install and
-** bootstrap the window.fossil object, using the built-in file
-** fossil.bootstrap.js (not to be confused with bootstrap.js).
+** Generate a <script> with an appropriate nonce.
 **
-** Subsequent calls are no-ops.
-**
-** It emits 2 parts:
-**
-** 1) window.fossil core object, some of which depends on C-level
-** runtime data. That part of the script is always emitted inline. If
-** addScriptTag is true then it is wrapped in its own SCRIPT tag, else
-** it is assumed that the caller already opened a tag.
-**
-** 2) Emits the static fossil.bootstrap.js using builtin_request_js().
+** zOrigin and iLine are the source code filename and line number
+** that generated this request.
 */
-void style_emit_script_fossil_bootstrap(int addScriptTag){
-  static int once = 0;
-  if(0==once++){
-    char * zName;
-    /* Set up the generic/app-agnostic parts of window.fossil
-    ** which require C-level state... */
-    if(addScriptTag!=0){
-      style_emit_script_tag(0,0);
+void style_script_begin(const char *zOrigin, int iLine){
+  const char *z;
+  for(z=zOrigin; z[0]!=0; z++){
+    if( z[0]=='/' || z[0]=='\\' ){
+      zOrigin = z+1;
     }
-    CX("(function(){\n");
-    CX(/*MSIE NodeList.forEach polyfill, courtesy of Mozilla:
-    https://developer.mozilla.org/en-US/docs/Web/API/NodeList/forEach#Polyfill
-       */
-       "if(window.NodeList && !NodeList.prototype.forEach){"
-       "NodeList.prototype.forEach = Array.prototype.forEach;"
-       "}\n");
-    CX("if(!window.fossil) window.fossil={};\n"
-       "window.fossil.version = %!j;\n"
-    /* fossil.rootPath is the top-most CGI/server path,
-    ** including a trailing slash. */
-       "window.fossil.rootPath = %!j+'/';\n",
-       get_version(), g.zTop);
-    /* fossil.config = {...various config-level options...} */
-    CX("window.fossil.config = {");
-    zName = db_get("project-name", "");
-    CX("projectName: %!j,\n", zName);
-    fossil_free(zName);
-    zName = db_get("short-project-name", "");
-    CX("shortProjectName: %!j,\n", zName);
-    fossil_free(zName);
-    zName = db_get("project-code", "");
-    CX("projectCode: %!j,\n", zName);
-    fossil_free(zName);
-    CX("/* Length of UUID hashes for display purposes. */");
-    CX("hashDigits: %d, hashDigitsUrl: %d,\n",
-       hash_digits(0), hash_digits(1));
-    CX("editStateMarkers: {"
-       "/*Symbolic markers to denote certain edit states.*/"
-       "isNew:'[+]', isModified:'[*]', isDeleted:'[-]'},\n");
-    CX("confirmerButtonTicks: 3 "
-       "/*default fossil.confirmer tick count.*/\n");
-    CX("};\n"/* fossil.config */);
-#if 0
-    /* Is it safe to emit the CSRF token here? Some pages add it
-    ** as a hidden form field. */
-    if(g.zCsrfToken[0]!=0){
-      CX("window.fossil.csrfToken = %!j;\n",
-         g.zCsrfToken);
-    }
-#endif
-    /*
-    ** fossil.page holds info about the current page. This is also
-    ** where the current page "should" store any of its own
-    ** page-specific state, and it is reserved for that purpose.
-    */
-    CX("window.fossil.page = {"
-       "name:\"%T\""
-       "};\n", g.zPath);
-    CX("})();\n");
-    if(addScriptTag!=0){
-      style_emit_script_tag(1,0);
-    }
-    /* The remaining window.fossil bootstrap code is not dependent on
-    ** C-runtime state... */
-    builtin_request_js("fossil.bootstrap.js");
   }
+  CX("<script nonce='%s'>/* %s:%d */\n", style_nonce(), zOrigin, iLine);
+}
+
+/* Generate the closing </script> tag 
+*/
+void style_script_end(void){
+  CX("</script>\n");
 }
 
 /*
-** If passed 0 as its first argument, it emits a script opener tag
-** with this request's nonce. If passed non-0 it emits a script
-** closing tag. Mnemonic for remembering the order in which to pass 0
-** or 1 as the first argument to this function: 0 comes before 1.
-**
-** If passed 0 as its first argument and a non-NULL/non-empty zSrc,
-** then it instead emits:
-**
-** <script src='%R/{{zSrc}}'></script>
-**
-** zSrc is always assumed to be a repository-relative path without
-** a leading slash, and has %R/ prepended to it.
-**
-** Meaning that no follow-up call to pass a non-0 first argument
-** to close the tag. zSrc is ignored if the first argument is not
-** 0.
+** Emits a NOSCRIPT tag with an error message stating that JS is
+** required for the current page. This "should" be called near the top
+** of pages which *require* JS. The inner DIV has the CSS class
+** 'error' and can be styled via a (noscript > .error) CSS selector.
 */
-void style_emit_script_tag(int isCloser, const char * zSrc){
-  if(0==isCloser){
-    if(zSrc!=0 && zSrc[0]!=0){
-      CX("<script src='%R/%T'></script>\n", zSrc);
-    }else{
-      CX("<script nonce='%s'>", style_nonce());
-    }
-  }else{
-    CX("</script>\n");
-  }
-}
-
-/*
-** Convenience wrapper which calls builtin_request_js() for a series
-** of builtin scripts named fossil.NAME.js. The first time it is
-** called, it also calls style_emit_script_fossil_bootstrap() to
-** initialize the window.fossil JS API. The first argument is a
-** no-meaning dummy required by the va_start() interface. All
-** subsequent arguments must be strings of the NAME part of
-** fossil.NAME.js, followed by a NULL argument to terminate the list.
-**
-** e.g. pass it (0, "fetch", "dom", "tabs", 0) to load those 3
-** APIs. Do not forget the trailing 0!
-*/
-void style_emit_fossil_js_apis( int dummy, ... ) {
-  static int once = 0;
-  const char *zArg;
-  char * zName;
-  va_list vargs;
-
-  if(0==once++){
-    style_emit_script_fossil_bootstrap(1);
-  }
-  va_start(vargs,dummy);
-  while( (zArg = va_arg (vargs, const char *))!=0 ){
-    zName = mprintf("fossil.%s.js", zArg);
-    builtin_request_js(zName);
-    fossil_free(zName);
-  }
-  va_end(vargs);
+void style_emit_noscript_for_js_page(void){
+  CX("<noscript><div class='error'>"
+     "This page requires JavaScript (ES2015, a.k.a. ES6, or newer)."
+     "</div></noscript>");
 }
