@@ -352,53 +352,56 @@ void sync_unversioned(unsigned syncFlags){
 **
 ** Usage: %fossil remote ?SUBCOMMAND ...?
 **
-** Use this command to view or modify the set of remote repositories
-** used as the default target for sync, push, and pull and for autosync.
+** View or modify the set of remote repository sync URLs used as the
+** target in any command that uses the sync protocol: "sync", "push",
+** and "pull", plus all other commands that trigger Fossil's autosync
+** feature.  (Collectively, "sync operations".)
 **
-** The default remote is set automatically by a "clone" command or by any
-** "sync", "push", or "pull" command that specifies an explicit URL
-** and omits the --once flag.  The default remote is used by
-** auto-syncing and by "sync", "push", and "pull" that omit the server URL.
-** Additional remotes can be added using the "add" command or deleted
-** using the "delete" command.  The name of any additional remote can be
-** used as an argument to the "sync", "push", and "pull" commands where
-** one would normally put a URL argument.
+** See "fossil help clone" for the format of these sync URLs.
 **
-** See "fossil help clone" for further information about URL formats.
+** Fossil implicitly sets the default remote sync URL from the initial
+** "clone" or "open URL" command for a repository, then may subsequently
+** change it when given a URL in commands that take a sync URL, except
+** when given the --once flag.  Fossil uses this new sync URL as its
+** default when not explicitly given one in subsequent sync operations.
 **
-** The official name of this command is "remote-url" but most people
-** use the shortened name "remote".
+** Named remotes added by "remote add" allow use of those names in place
+** of a sync URL in any command that takes one.
+**
+** The full name of this command is "remote-url", but we anticipate no
+** future collision from use of its shortened form "remote".
 **
 ** > fossil remote
 **
-**     With no arguments, this command shows the current default remote.
-**     Or if there is no default, it shows "off".  The default remote is
-**     used by autosync.  The default remote is whatever URL was specified
-**     for the most recent "sync", "push", or "pull" command that omitted
-**     the --once option.
+**     With no arguments, this command shows the current default remote
+**     URL.  If there is no default, it shows "off".
 **
 ** > fossil remote add NAME URL
 **
-**     Add a new URL to the set of remotes.  The new URL is assigned the
-**     symbolic identifier "NAME".  Subsequently, NAME can be used in place
-**     of the full URL on commands like "push" and "pull".
+**     Add a new named URL to the set of remote sync URLs for use in
+**     place of a sync URL in commands that take one.
 **
 ** > fossil remote delete NAME
 **
-**     Delete a URL previously added by the "add" subcommand.
+**     Delete a sync URL previously added by the "add" subcommand.
 **
 ** > fossil remote list
 **
-**     Show all remote URLs
+**     Show all remote repository sync URLs.
 **
 ** > fossil remote off
 **
-**     Disable the default URL.  Use this as a shorthand to prevent
-**     autosync while in airplane mode, for example.
+**     Forget the default sync URL, disabling autosync.  Combined with
+**     named sync URLs, it allows canceling this "airplane mode" with
+**     "fossil remote NAME" to select a previously-set named URL.
 **
-** > fossil remote URL
+**     To disable use of the default remote without forgetting its URL,
+**     say "fossil set autosync 0" instead.
 **
-**     Make URL the new default URL.  The prior default URL is replaced.
+** > fossil remote REF
+**
+**     Make REF the new default URL, replacing the prior default.
+**     REF may be a URL or a NAME from a prior "add".
 */
 void remote_url_cmd(void){
   char *zUrl, *zArg;
@@ -427,9 +430,11 @@ void remote_url_cmd(void){
     */
     if( g.argc!=3 ) usage("off");
 remote_delete_default:
+    db_unprotect(PROTECT_CONFIG);
     db_multi_exec(
       "DELETE FROM config WHERE name GLOB 'last-sync-*';"
     );
+    db_protect_pop();
     return;
   }
   if( strncmp(zArg, "list", nArg)==0 || strcmp(zArg,"ls")==0 ){
@@ -459,6 +464,7 @@ remote_delete_default:
     if( strcmp(zName,"default")==0 ) goto remote_add_default;
     url_parse_local(zUrl, URL_PROMPT_PW, &x);
     db_begin_write();
+    db_unprotect(PROTECT_CONFIG);
     db_multi_exec(
        "REPLACE INTO config(name, value, mtime)"
        " VALUES('sync-url:%q',%Q,now())",
@@ -469,6 +475,7 @@ remote_delete_default:
        " VALUES('sync-pw:%q',obscure(%Q),now())",
        zName, x.passwd
     );
+    db_protect_pop();
     db_commit_transaction();
     return;
   }
@@ -478,8 +485,10 @@ remote_delete_default:
     zName = g.argv[3];
     if( strcmp(zName,"default")==0 ) goto remote_delete_default;
     db_begin_write();
+    db_unprotect(PROTECT_CONFIG);
     db_multi_exec("DELETE FROM config WHERE name glob 'sync-url:%q'", zName);
     db_multi_exec("DELETE FROM config WHERE name glob 'sync-pw:%q'", zName);
+    db_protect_pop();
     db_commit_transaction();
     return;
   }
@@ -507,7 +516,7 @@ remote_add_default:
 **
 ** Make a backup of the repository into the named file or into the named
 ** directory.  This backup is guaranteed to be consistent even if there are
-** concurrent chnages taking place on the repository.  In other words, it
+** concurrent changes taking place on the repository.  In other words, it
 ** is safe to run "fossil backup" on a repository that is in active use.
 **
 ** Only the main repository database is backed up by this command.  The
@@ -516,7 +525,7 @@ remote_add_default:
 **
 ** Options:
 **
-**    --overwrite              OK to overwrite an existing file.
+**    --overwrite              OK to overwrite an existing file
 **    -R NAME                  Filename of the repository to backup
 */
 void backup_cmd(void){
@@ -541,5 +550,6 @@ void backup_cmd(void){
       fossil_fatal("backup \"%s\" already exists", zDest);
     }
   }
+  db_unprotect(PROTECT_ALL);
   db_multi_exec("VACUUM repository INTO %Q", zDest);
 }
