@@ -529,7 +529,7 @@ void tarball_of_checkin(
           zName = blob_str(&filename);
         }
         if( eflg & MFESTFLG_RAW ) {
-          sterilize_manifest(&mfile);
+          sterilize_manifest(&mfile, CFTYPE_MANIFEST);
           tar_add_file(zName, &mfile, 0, mTime);
         }
       }
@@ -787,7 +787,7 @@ void tarball_page(void){
       @ zExclude = "%h(zExclude)"<br />
     }
     @ zKey = "%h(zKey)"
-    style_footer();
+    style_finish_page();
     return;
   }
   if( referred_from_login() ){
@@ -798,7 +798,7 @@ void tarball_page(void){
     @ of check-in <b>%h(zRid)</b>:
     @ <input type="submit" value="Download" />
     @ </form>
-    style_footer();
+    style_finish_page();
     return;
   }
   blob_zero(&tarball);
