@@ -541,10 +541,12 @@ void win32_http_server(
     blob_appendf(&options, " --https");
   }
   if( zBaseUrl ){
-    blob_appendf(&options, " --baseurl %s", zBaseUrl);
+    blob_appendf(&options, " --baseurl ");
+    blob_append_escaped_arg(&options, zBaseUrl);
   }
   if( zNotFound ){
-    blob_appendf(&options, " --notfound %s", zNotFound);
+    blob_appendf(&options, " --notfound ");
+    blob_append_escaped_arg(&options, zNotFound);
   }
   if( zFileGlob ){
     blob_appendf(&options, " --files-urlenc %T", zFileGlob);
@@ -558,6 +560,10 @@ void win32_http_server(
   if( flags & HTTP_SERVER_REPOLIST ){
     blob_appendf(&options, " --repolist");
   }
+  if( g.zExtRoot && g.zExtRoot[0] ){
+    blob_appendf(&options, " --extroot");
+    blob_append_escaped_arg(&options, g.zExtRoot);
+  }
   zSkin = skin_in_use();
   if( zSkin ){
     blob_appendf(&options, " --skin %s", zSkin);
