@@ -681,7 +681,7 @@ static void zip_of_checkin(
         blob_append(&filename, "manifest", -1);
         zName = blob_str(&filename);
         zip_add_folders(&sArchive, zName);
-        sterilize_manifest(&mfile);
+        sterilize_manifest(&mfile, CFTYPE_MANIFEST);
         zip_add_file(&sArchive, zName, &mfile, 0);
       }
       if( eflg & MFESTFLG_UUID ){
@@ -946,6 +946,7 @@ void baseline_zip_page(void){
   zKey = blob_str(&cacheKey);
   etag_check(ETAG_HASH, zKey);
 
+  style_set_current_feature("zip");
   if( P("debug")!=0 ){
     style_header("%s Archive Generator Debug Screen", zType);
     @ zName = "%h(zName)"<br />
@@ -957,7 +958,7 @@ void baseline_zip_page(void){
       @ zExclude = "%h(zExclude)"<br />
     }
     @ zKey = "%h(zKey)"
-    style_footer();
+    style_finish_page();
     return;
   }
   if( referred_from_login() ){
@@ -968,7 +969,7 @@ void baseline_zip_page(void){
     @ holding the content of check-in <b>%h(zRid)</b>:
     @ <input type="submit" value="Download" />
     @ </form>
-    style_footer();
+    style_finish_page();
     return;
   }
   blob_zero(&zip);
